import _socket
import email.message
import io
import socketserver
import sys
from _ssl import _PasswordType
from _typeshed import ReadableBuffer, StrOrBytesPath, StrPath, SupportsRead, SupportsWrite
from collections.abc import Callable, Iterable, Mapping, Sequence
from ssl import Purpose, SSLContext
from typing import Any, AnyStr, BinaryIO, ClassVar, Protocol, type_check_only
from typing_extensions import Self, deprecated

__all__ = ["HTTPServer", "ThreadingHTTPServer", "BaseHTTPRequestHandler", "SimpleHTTPRequestHandler"]
if sys.version_info < (3, 15):
    __all__ += ["CGIHTTPRequestHandler"]
if sys.version_info >= (3, 14):
    __all__ = ["HTTPSServer", "ThreadingHTTPSServer"]

class HTTPServer(socketserver.TCPServer):
    server_name: str
    server_port: int

class ThreadingHTTPServer(socketserver.ThreadingMixIn, HTTPServer): ...

if sys.version_info >= (3, 14):
    @type_check_only
    class _SSLModule(Protocol):
        @staticmethod
        def create_default_context(
            purpose: Purpose = ...,
            *,
            cafile: StrOrBytesPath | None = None,
            capath: StrOrBytesPath | None = None,
            cadata: str | ReadableBuffer | None = None,
        ) -> SSLContext: ...

    class HTTPSServer(HTTPServer):
        ssl: _SSLModule
        certfile: StrOrBytesPath
        keyfile: StrOrBytesPath | None
        password: _PasswordType | None
        alpn_protocols: Iterable[str]
        def __init__(
            self,
            server_address: socketserver._AfInetAddress,
            RequestHandlerClass: Callable[[Any, _socket._RetAddress, Self], socketserver.BaseRequestHandler],
            bind_and_activate: bool = True,
            *,
            certfile: StrOrBytesPath,
            keyfile: StrOrBytesPath | None = None,
            password: _PasswordType | None = None,
            alpn_protocols: Iterable[str] | None = None,
        ) -> None: ...
        def server_activate(self) -> None: ...

    class ThreadingHTTPSServer(socketserver.ThreadingMixIn, HTTPSServer): ...

class BaseHTTPRequestHandler(socketserver.StreamRequestHandler):
    client_address: tuple[str, int]
    close_connection: bool
    requestline: str
    command: str
    path: str
    request_version: str
    headers: email.message.Message
    server_version: str
    sys_version: str
    error_message_format: str
    error_content_type: str
    protocol_version: str
    MessageClass: type
    responses: Mapping[int, tuple[str, str]]
    if sys.version_info >= (3, 15):
        default_content_type: str
    default_request_version: str  # undocumented
    weekdayname: ClassVar[Sequence[str]]  # undocumented
    monthname: ClassVar[Sequence[str | None]]  # undocumented
    def handle_one_request(self) -> None: ...
    def handle_expect_100(self) -> bool: ...
    def send_error(self, code: int, message: str | None = None, explain: str | None = None) -> None: ...
    def send_response(self, code: int, message: str | None = None) -> None: ...
    def send_header(self, keyword: str, value: str) -> None: ...
    def send_response_only(self, code: int, message: str | None = None) -> None: ...
    def end_headers(self) -> None: ...
    def flush_headers(self) -> None: ...
    def log_request(self, code: int | str = "-", size: int | str = "-") -> None: ...
    def log_error(self, format: str, *args: Any) -> None: ...
    def log_message(self, format: str, *args: Any) -> None: ...
    def version_string(self) -> str: ...
    def date_time_string(self, timestamp: float | None = None) -> str: ...
    def log_date_time_string(self) -> str: ...
    def address_string(self) -> str: ...
    def parse_request(self) -> bool: ...  # undocumented

class SimpleHTTPRequestHandler(BaseHTTPRequestHandler):
    extensions_map: dict[str, str]
    if sys.version_info >= (3, 12):
        index_pages: ClassVar[tuple[str, ...]]
    directory: str
    if sys.version_info >= (3, 15):
        def __init__(
            self,
            request: socketserver._RequestType,
            client_address: _socket._RetAddress,
            server: socketserver.BaseServer,
            *,
            directory: StrPath | None = None,
            extra_response_headers: Mapping[str, str] | None = None,
        ) -> None: ...
    else:
        def __init__(
            self,
            request: socketserver._RequestType,
            client_address: _socket._RetAddress,
            server: socketserver.BaseServer,
            *,
            directory: StrPath | None = None,
        ) -> None: ...

    def do_GET(self) -> None: ...
    def do_HEAD(self) -> None: ...
    def send_head(self) -> io.BytesIO | BinaryIO | None: ...  # undocumented
    def list_directory(self, path: StrPath) -> io.BytesIO | None: ...  # undocumented
    def translate_path(self, path: str) -> str: ...  # undocumented
    def copyfile(self, source: SupportsRead[AnyStr], outputfile: SupportsWrite[AnyStr]) -> None: ...  # undocumented
    def guess_type(self, path: StrPath) -> str: ...  # undocumented

def executable(path: StrPath) -> bool: ...  # undocumented

if sys.version_info < (3, 15):
    @deprecated("Deprecated since Python 3.13; will be removed in Python 3.15.")
    class CGIHTTPRequestHandler(SimpleHTTPRequestHandler):
        cgi_directories: list[str]
        have_fork: bool  # undocumented
        def do_POST(self) -> None: ...
        def is_cgi(self) -> bool: ...  # undocumented
        def is_executable(self, path: StrPath) -> bool: ...  # undocumented
        def is_python(self, path: StrPath) -> bool: ...  # undocumented
        def run_cgi(self) -> None: ...  # undocumented



import enum
import sys
from _typeshed import Unused
from collections import deque
from collections.abc import Callable
from types import TracebackType
from typing import Any, Literal, TypeVar
from typing_extensions import Self

from .futures import Future
from .mixins import _LoopBoundMixin

# Keep asyncio.__all__ updated with any changes to __all__ here
if sys.version_info >= (3, 11):
    __all__ = ("Lock", "Event", "Condition", "Semaphore", "BoundedSemaphore", "Barrier")
else:
    __all__ = ("Lock", "Event", "Condition", "Semaphore", "BoundedSemaphore")

_T = TypeVar("_T")

class _ContextManagerMixin:
    async def __aenter__(self) -> None: ...
    async def __aexit__(
        self, exc_type: type[BaseException] | None, exc: BaseException | None, tb: TracebackType | None
    ) -> None: ...

class Lock(_ContextManagerMixin, _LoopBoundMixin):
    _waiters: deque[Future[Any]] | None
    def __init__(self) -> None: ...
    def locked(self) -> bool: ...
    async def acquire(self) -> Literal[True]: ...
    def release(self) -> None: ...

class Event(_LoopBoundMixin):
    _waiters: deque[Future[Any]]
    def __init__(self) -> None: ...
    def is_set(self) -> bool: ...
    def set(self) -> None: ...
    def clear(self) -> None: ...
    async def wait(self) -> Literal[True]: ...

class Condition(_ContextManagerMixin, _LoopBoundMixin):
    _waiters: deque[Future[Any]]
    def __init__(self, lock: Lock | None = None) -> None: ...
    def locked(self) -> bool: ...
    async def acquire(self) -> Literal[True]: ...
    def release(self) -> None: ...
    async def wait(self) -> Literal[True]: ...
    async def wait_for(self, predicate: Callable[[], _T]) -> _T: ...
    def notify(self, n: int = 1) -> None: ...
    def notify_all(self) -> None: ...

class Semaphore(_ContextManagerMixin, _LoopBoundMixin):
    _value: int
    _waiters: deque[Future[Any]] | None
    def __init__(self, value: int = 1) -> None: ...
    def locked(self) -> bool: ...
    async def acquire(self) -> Literal[True]: ...
    def release(self) -> None: ...
    def _wake_up_next(self) -> None: ...

class BoundedSemaphore(Semaphore): ...

if sys.version_info >= (3, 11):
    class _BarrierState(enum.Enum):  # undocumented
        FILLING = "filling"
        DRAINING = "draining"
        RESETTING = "resetting"
        BROKEN = "broken"

    class Barrier(_LoopBoundMixin):
        def __init__(self, parties: int) -> None: ...
        async def __aenter__(self) -> Self: ...
        async def __aexit__(self, *args: Unused) -> None: ...
        async def wait(self) -> int: ...
        async def abort(self) -> None: ...
        async def reset(self) -> None: ...
        @property
        def parties(self) -> int: ...
        @property
        def n_waiting(self) -> int: ...
        @property
        def broken(self) -> bool: ...



from collections.abc import Generator
from pathlib import Path
from typing import SupportsInt, TypeAlias
from typing_extensions import Self

__all__ = ("LANGUAGES", "Pyphen", "language_fallback")
LANGUAGES: dict[str, Path]
_Data: TypeAlias = tuple[str, int, int]

def language_fallback(language: str) -> str: ...

class AlternativeParser:
    change: str
    index: int
    cut: int

    def __init__(self, pattern: str, alternative: str) -> None: ...
    def __call__(self, value: str | SupportsInt) -> int: ...

class DataInt(int):
    data: _Data | None

    def __new__(cls, value: int, data: _Data | None = ..., reference: DataInt | None = ...) -> Self: ...

class HyphDict:
    patterns: dict[str, tuple[int, tuple[int, ...]]]
    cache: dict[str, list[DataInt]]
    maxlen: int

    def __init__(self, path: Path) -> None: ...
    def positions(self, word: str) -> list[DataInt]: ...

class Pyphen:
    hd: HyphDict

    def __init__(
        self, filename: str | Path | None = ..., lang: str | None = ..., left: int = 2, right: int = 2, cache: bool = True
    ) -> None: ...
    def positions(self, word: str) -> list[DataInt]: ...
    def iterate(self, word: str) -> Generator[tuple[str, str]]: ...
    def wrap(self, word: str, width: int, hyphen: str = "-") -> tuple[str, str] | None: ...
    def inserted(self, word: str, hyphen: str = "-") -> str: ...
    __call__ = iterate



from _typeshed import Incomplete

from kafka.protocol.api import Request, Response

class EndTxnResponse_v0(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class EndTxnResponse_v1(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class EndTxnResponse_v2(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class EndTxnRequest_v0(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = EndTxnResponse_v0
    SCHEMA: Incomplete

class EndTxnRequest_v1(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = EndTxnResponse_v1
    SCHEMA: Incomplete

class EndTxnRequest_v2(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = EndTxnResponse_v2
    SCHEMA: Incomplete

EndTxnRequest: Incomplete
EndTxnResponse: Incomplete



from _typeshed import Incomplete, Unused
from collections.abc import Iterator
from typing import Final, TypeAlias

import _win32typing
from pythoncom import com_record
from win32.lib.pywintypes import IIDType
from win32com.client import dynamic

_Stringifiable: TypeAlias = object

def GetObject(Pathname: str | None = None, Class=None, clsctx=None) -> CDispatch: ...
def GetActiveObject(Class: str, clsctx=...): ...
def Moniker(Pathname: str, clsctx=...): ...
def Dispatch(
    dispatch: str | dynamic.PyIDispatchType | IIDType | dynamic.PyIUnknownType,
    userName: str | None = None,
    resultCLSID: _Stringifiable | None = None,
    typeinfo: _win32typing.PyITypeInfo | None = None,
    clsctx: int = ...,
) -> dynamic.CDispatch: ...
def DispatchEx(clsid, machine=None, userName=None, resultCLSID=None, typeinfo=None, clsctx=None): ...

class CDispatch(dynamic.CDispatch):
    def __dir__(self) -> list[str]: ...

def CastTo(ob, target, typelib=None): ...

class Constants:
    __dicts__: list[Incomplete]
    def __getattr__(self, a: str): ...

constants: Final[Constants]

class EventsProxy:
    def __init__(self, ob) -> None: ...
    def __del__(self) -> None: ...
    def __getattr__(self, attr: str): ...
    def __setattr__(self, attr: str, val) -> None: ...

def DispatchWithEvents(clsid, user_event_class) -> EventsProxy: ...
def WithEvents(disp, user_event_class): ...
def getevents(clsid): ...
def Record(name, object) -> com_record: ...
def register_record_class(cls) -> None: ...

class DispatchBaseClass:
    def __init__(self, oobj=None) -> None: ...
    def __dir__(self) -> list[str]: ...
    def __eq__(self, other) -> bool: ...
    def __ne__(self, other) -> bool: ...
    def __getattr__(self, attr: str): ...
    def __setattr__(self, attr: str, value) -> None: ...

class CoClassBaseClass:
    def __init__(self, oobj=None) -> None: ...
    def __getattr__(self, attr: str): ...
    def __setattr__(self, attr: str, value) -> None: ...
    def __call__(self, *args, **kwargs): ...
    def __str__(self, *args: Unused) -> str: ...  # noqa: Y029
    def __int__(self, *args: Unused) -> int: ...
    def __iter__(self) -> Iterator[Incomplete]: ...
    def __len__(self) -> int: ...
    def __bool__(self) -> bool: ...

class VARIANT:
    varianttype: Incomplete
    def __init__(self, vt, value) -> None: ...

    @property
    def value(self): ...
    @value.setter
    def value(self, newval) -> None: ...
    @value.deleter
    def value(self) -> None: ...



from _typeshed import Incomplete
from collections.abc import Callable
from contextlib import AbstractContextManager
from typing import ClassVar, ParamSpec

_P = ParamSpec("_P")

class Context:
    LAYER_NAMES: ClassVar[list[str]]
    FAIL_ON_CLEANUP_ERRORS: ClassVar[bool]

    feature: Incomplete | None
    scenario: Incomplete
    tags: set[str]
    aborted: bool
    failed: bool
    table: Incomplete | None
    text: str | None
    config: Incomplete
    active_outline: Incomplete
    fail_on_cleanup_errors: bool

    def __init__(self, runner) -> None: ...
    def __getattr__(self, name: str) -> Incomplete: ...
    def __setattr__(self, name: str, value) -> None: ...
    def __delattr__(self, name: str) -> None: ...
    def __contains__(self, name: str) -> bool: ...
    def abort(self, reason: str | None = None) -> None: ...
    def use_or_assign_param(self, name: str, value): ...
    def use_or_create_param(self, name: str, factory_func: Callable[_P, Incomplete], *args: _P.args, **kwargs: _P.kwargs): ...
    def use_with_user_mode(self) -> AbstractContextManager[None]: ...
    def execute_steps(self, steps_text: str) -> bool: ...
    def add_cleanup(self, cleanup_func: Callable[_P, Incomplete], *args: _P.args, **kwargs: _P.kwargs) -> None: ...
    @property
    def captured(self): ...
    def attach(self, mime_type: str, data: bytes) -> None: ...

def __getattr__(name: str) -> Incomplete: ...



from collections.abc import Mapping, Sequence
from typing import Any

from .descriptor_pb2 import (
    DescriptorProto,
    EnumDescriptorProto,
    EnumOptions,
    EnumValueOptions,
    FieldOptions,
    FileDescriptorProto,
    FileOptions,
    MessageOptions,
    MethodDescriptorProto,
    MethodOptions,
    OneofOptions,
    ServiceDescriptorProto,
    ServiceOptions,
)
from .descriptor_pool import DescriptorPool
from .message import Message

class Error(Exception): ...
class TypeTransformationError(Error): ...

class DescriptorMetaclass(type):
    def __instancecheck__(cls, obj: object) -> bool: ...

_internal_create_key: object
_USE_C_DESCRIPTORS: bool

class DescriptorBase(metaclass=DescriptorMetaclass):
    has_options: bool
    def __init__(self, file, options, serialized_options, options_class_name) -> None: ...
    def GetOptions(self) -> Any: ...  # Any: overridden with specific *Options in subclasses

class _NestedDescriptorBase(DescriptorBase):
    name: str
    full_name: str
    file: FileDescriptor
    containing_type: Descriptor | None
    def __init__(
        self,
        options,
        options_class_name,
        name,
        full_name,
        file,
        containing_type,
        serialized_start=None,
        serialized_end=None,
        serialized_options=None,
    ) -> None: ...
    def CopyToProto(self, proto: Any) -> None: ...  # Any: overridden with specific *Proto in subclasses

class Descriptor(_NestedDescriptorBase):
    fields: Sequence[FieldDescriptor]
    fields_by_number: Mapping[int, FieldDescriptor]
    fields_by_name: Mapping[str, FieldDescriptor]
    @property
    def fields_by_camelcase_name(self) -> Mapping[str, FieldDescriptor]: ...
    nested_types: Sequence[Descriptor]
    nested_types_by_name: Mapping[str, Descriptor]
    enum_types: Sequence[EnumDescriptor]
    enum_types_by_name: Mapping[str, EnumDescriptor]
    enum_values_by_name: dict[str, EnumValueDescriptor]
    extensions: Sequence[FieldDescriptor]
    extensions_by_name: Mapping[str, FieldDescriptor]
    is_extendable: bool
    extension_ranges: list[tuple[int, int]]
    oneofs: Sequence[OneofDescriptor]
    oneofs_by_name: Mapping[str, OneofDescriptor]
    def __init__(
        self,
        name: str,
        full_name: str,
        filename: str | None,
        containing_type: Descriptor | None,
        fields: list[FieldDescriptor],
        nested_types: list[FieldDescriptor],
        enum_types: list[EnumDescriptor],
        extensions: list[FieldDescriptor],
        options: MessageOptions | None = None,
        serialized_options: bytes | None = None,
        is_extendable: bool | None = True,
        extension_ranges: list[tuple[int, int]] | None = None,
        oneofs: list[OneofDescriptor] | None = None,
        file: FileDescriptor | None = None,
        serialized_start: int | None = None,
        serialized_end: int | None = None,
        syntax: str | None = None,
        is_map_entry: bool = False,
        create_key: object | None = None,
    ): ...
    def EnumValueName(self, enum: str, value: int) -> str: ...
    def CopyToProto(self, proto: DescriptorProto) -> None: ...
    def GetOptions(self) -> MessageOptions: ...

class FieldDescriptor(DescriptorBase):
    TYPE_DOUBLE: int
    TYPE_FLOAT: int
    TYPE_INT64: int
    TYPE_UINT64: int
    TYPE_INT32: int
    TYPE_FIXED64: int
    TYPE_FIXED32: int
    TYPE_BOOL: int
    TYPE_STRING: int
    TYPE_GROUP: int
    TYPE_MESSAGE: int
    TYPE_BYTES: int
    TYPE_UINT32: int
    TYPE_ENUM: int
    TYPE_SFIXED32: int
    TYPE_SFIXED64: int
    TYPE_SINT32: int
    TYPE_SINT64: int
    MAX_TYPE: int
    CPPTYPE_INT32: int
    CPPTYPE_INT64: int
    CPPTYPE_UINT32: int
    CPPTYPE_UINT64: int
    CPPTYPE_DOUBLE: int
    CPPTYPE_FLOAT: int
    CPPTYPE_BOOL: int
    CPPTYPE_ENUM: int
    CPPTYPE_STRING: int
    CPPTYPE_MESSAGE: int
    MAX_CPPTYPE: int
    LABEL_OPTIONAL: int
    LABEL_REQUIRED: int
    LABEL_REPEATED: int
    MAX_LABEL: int
    MAX_FIELD_NUMBER: int
    FIRST_RESERVED_FIELD_NUMBER: int
    LAST_RESERVED_FIELD_NUMBER: int
    def __new__(
        cls,
        name,
        full_name,
        index,
        number,
        type,
        cpp_type,
        label,
        default_value,
        message_type,
        enum_type,
        containing_type,
        is_extension,
        extension_scope,
        options=None,
        serialized_options=None,
        has_default_value=True,
        containing_oneof=None,
        json_name=None,
        file=None,
        create_key=None,
    ): ...
    name: str
    full_name: str
    index: int
    number: int
    type: int
    cpp_type: int
    @property
    def is_required(self) -> bool: ...
    @property
    def is_repeated(self) -> bool: ...
    @property
    def camelcase_name(self) -> str: ...
    @property
    def has_presence(self) -> bool: ...
    @property
    def is_packed(self) -> bool: ...
    has_default_value: bool
    default_value: Any  # Any: str, int, float, bytes, or bool
    containing_type: Descriptor | None
    message_type: Descriptor | None
    enum_type: EnumDescriptor | None
    is_extension: bool
    extension_scope: Descriptor | None
    containing_oneof: OneofDescriptor | None
    json_name: str
    def __init__(
        self,
        name,
        full_name,
        index,
        number,
        type,
        cpp_type,
        label,
        default_value,
        message_type,
        enum_type,
        containing_type,
        is_extension,
        extension_scope,
        options=None,
        serialized_options=None,
        has_default_value=True,
        containing_oneof=None,
        json_name=None,
        file=None,
        create_key=None,
    ) -> None: ...
    @staticmethod
    def ProtoTypeToCppProtoType(proto_type: int) -> int: ...
    def GetOptions(self) -> FieldOptions: ...

class EnumDescriptor(_NestedDescriptorBase):
    def __new__(
        cls,
        name,
        full_name,
        filename,
        values,
        containing_type=None,
        options=None,
        serialized_options=None,
        file=None,
        serialized_start=None,
        serialized_end=None,
        create_key=None,
    ): ...
    values: Sequence[EnumValueDescriptor]
    values_by_name: Mapping[str, EnumValueDescriptor]
    values_by_number: Mapping[int, EnumValueDescriptor]
    def __init__(
        self,
        name,
        full_name,
        filename,
        values,
        containing_type=None,
        options=None,
        serialized_options=None,
        file=None,
        serialized_start=None,
        serialized_end=None,
        create_key=None,
    ) -> None: ...
    @property
    def is_closed(self) -> bool: ...
    def CopyToProto(self, proto: EnumDescriptorProto) -> None: ...
    def GetOptions(self) -> EnumOptions: ...

class EnumValueDescriptor(DescriptorBase):
    def __new__(cls, name, index, number, type=None, options=None, serialized_options=None, create_key=None): ...
    name: str
    index: int
    number: int
    type: EnumDescriptor
    def __init__(self, name, index, number, type=None, options=None, serialized_options=None, create_key=None) -> None: ...
    def GetOptions(self) -> EnumValueOptions: ...

class OneofDescriptor(DescriptorBase):
    def __new__(cls, name, full_name, index, containing_type, fields, options=None, serialized_options=None, create_key=None): ...
    name: str
    full_name: str
    index: int
    containing_type: Descriptor
    fields: Sequence[FieldDescriptor]
    def __init__(
        self, name, full_name, index, containing_type, fields, options=None, serialized_options=None, create_key=None
    ) -> None: ...
    def GetOptions(self) -> OneofOptions: ...

class ServiceDescriptor(_NestedDescriptorBase):
    index: int
    methods: Sequence[MethodDescriptor]
    methods_by_name: Mapping[str, MethodDescriptor]
    def __init__(
        self,
        name: str,
        full_name: str,
        index: int,
        methods: list[MethodDescriptor],
        options: ServiceOptions | None = None,
        serialized_options: bytes | None = None,
        file: FileDescriptor | None = None,
        serialized_start: int | None = None,
        serialized_end: int | None = None,
        create_key: object | None = None,
    ): ...
    def FindMethodByName(self, name: str) -> MethodDescriptor: ...
    def CopyToProto(self, proto: ServiceDescriptorProto) -> None: ...
    def GetOptions(self) -> ServiceOptions: ...

class MethodDescriptor(DescriptorBase):
    def __new__(
        cls,
        name,
        full_name,
        index,
        containing_service,
        input_type,
        output_type,
        client_streaming=False,
        server_streaming=False,
        options=None,
        serialized_options=None,
        create_key=None,
    ): ...
    name: str
    full_name: str
    index: int
    containing_service: ServiceDescriptor
    input_type: Descriptor
    output_type: Descriptor
    client_streaming: bool
    server_streaming: bool
    def __init__(
        self,
        name,
        full_name,
        index,
        containing_service,
        input_type,
        output_type,
        client_streaming=False,
        server_streaming=False,
        options=None,
        serialized_options=None,
        create_key=None,
    ) -> None: ...
    def CopyToProto(self, proto: MethodDescriptorProto) -> None: ...
    def GetOptions(self) -> MethodOptions: ...

class FileDescriptor(DescriptorBase):
    def __new__(
        cls,
        name,
        package,
        options=None,
        serialized_options=None,
        serialized_pb=None,
        dependencies=None,
        public_dependencies=None,
        syntax=None,
        edition=None,
        pool=None,
        create_key=None,
    ): ...
    _options: Any
    _loaded_options: Any
    pool: DescriptorPool
    message_types_by_name: Mapping[str, Descriptor]
    name: str
    package: str
    serialized_pb: bytes
    enum_types_by_name: Mapping[str, EnumDescriptor]
    extensions_by_name: Mapping[str, FieldDescriptor]
    services_by_name: Mapping[str, ServiceDescriptor]
    dependencies: Sequence[FileDescriptor]
    public_dependencies: Sequence[FileDescriptor]
    def __init__(
        self,
        name,
        package,
        options=None,
        serialized_options=None,
        serialized_pb=None,
        dependencies=None,
        public_dependencies=None,
        syntax=None,
        edition=None,
        pool=None,
        create_key=None,
    ) -> None: ...
    def CopyToProto(self, proto: FileDescriptorProto) -> None: ...
    def GetOptions(self) -> FileOptions: ...

def _ParseOptions(message: Message, string: bytes) -> Message: ...
def MakeDescriptor(
    desc_proto: DescriptorProto,
    package: str = "",
    build_file_if_cpp: bool = True,
    syntax: str | None = None,
    edition: str | None = None,
    file_desc: FileDescriptor | None = None,
) -> Descriptor: ...



from _typeshed import Incomplete, ReadableBuffer
from collections import OrderedDict
from collections.abc import Iterable
from typing import SupportsBytes, SupportsIndex

from .models import JWEAlgorithmBase, JWEEncAlgorithm, JWEZipAlgorithm

class JsonWebEncryption:
    REGISTERED_HEADER_PARAMETER_NAMES: frozenset[str]
    ALG_REGISTRY: dict[str, JWEAlgorithmBase]
    ENC_REGISTRY: dict[str, JWEEncAlgorithm]
    ZIP_REGISTRY: dict[str, JWEZipAlgorithm]
    def __init__(self, algorithms=None, private_headers=None) -> None: ...
    @classmethod
    def register_algorithm(cls, algorithm: JWEAlgorithmBase | JWEEncAlgorithm | JWEZipAlgorithm) -> None: ...
    def serialize_compact(self, protected, payload, key, sender_key=None) -> bytes: ...
    def serialize_json(self, header_obj, payload, keys, sender_key=None) -> OrderedDict[Incomplete, Incomplete]: ...
    def serialize(self, header, payload, key, sender_key=None) -> OrderedDict[Incomplete, Incomplete] | bytes: ...
    def deserialize_compact(
        self,
        s: str | bytes | float | Iterable[SupportsIndex] | SupportsIndex | SupportsBytes | ReadableBuffer,
        key,
        decode=None,
        sender_key=None,
    ) -> dict[str, Incomplete]: ...
    def deserialize_json(self, obj, key, decode=None, sender_key=None) -> dict[str, Incomplete]: ...
    def deserialize(self, obj, key, decode=None, sender_key=None) -> dict[str, Incomplete]: ...
    @staticmethod
    def parse_json(obj) -> dict[Incomplete, Incomplete]: ...
    def get_header_alg(self, header) -> JWEAlgorithmBase: ...
    def get_header_enc(self, header) -> JWEEncAlgorithm: ...
    def get_header_zip(self, header) -> JWEZipAlgorithm: ...

def prepare_key(alg, header, key): ...



from _typeshed import Incomplete
from collections.abc import Mapping, Sequence
from pathlib import Path
from typing import Any, Generic, Literal, ParamSpec, TypeAlias, TypeVar

import tensorflow as tf
from tensorflow.python.training.tracking.autotrackable import AutoTrackable
from tensorflow.saved_model.experimental import VariablePolicy
from tensorflow.types.experimental import ConcreteFunction, PolymorphicFunction

_P = ParamSpec("_P")
_R_co = TypeVar("_R_co", covariant=True)

class Asset:
    @property
    def asset_path(self) -> tf.Tensor: ...
    def __init__(self, path: str | Path | tf.Tensor) -> None: ...

class LoadOptions:
    __slots__ = (
        "allow_partial_checkpoint",
        "experimental_io_device",
        "experimental_skip_checkpoint",
        "experimental_variable_policy",
        "experimental_load_function_aliases",
    )
    allow_partial_checkpoint: bool
    experimental_io_device: str | None
    experimental_skip_checkpoint: bool
    experimental_variable_policy: VariablePolicy | None
    experimental_load_function_aliases: bool

    def __init__(
        self,
        allow_partial_checkpoint: bool = False,
        experimental_io_device: str | None = None,
        experimental_skip_checkpoint: bool = False,
        experimental_variable_policy: (
            VariablePolicy | Literal["expand_distributed_variables", "save_variable_devices"] | None
        ) = None,
        experimental_load_function_aliases: bool = False,
    ) -> None: ...

class SaveOptions:
    __slots__ = (
        "namespace_whitelist",
        "save_debug_info",
        "function_aliases",
        "experimental_debug_stripper",
        "experimental_io_device",
        "experimental_variable_policy",
        "experimental_custom_gradients",
        "experimental_image_format",
        "experimental_skip_saver",
        "experimental_sharding_callback",
        "extra_tags",
    )
    namespace_whitelist: list[str]
    save_debug_info: bool
    function_aliases: dict[str, PolymorphicFunction[..., object]]
    experimental_debug_stripper: bool
    experimental_io_device: str
    experimental_variable_policy: VariablePolicy
    experimental_custom_gradients: bool
    experimental_image_format: bool
    experimental_skip_saver: bool
    experimental_sharding_callback: Incomplete | None
    extra_tags: Incomplete | None
    def __init__(
        self,
        namespace_whitelist: list[str] | None = None,
        save_debug_info: bool = False,
        function_aliases: Mapping[str, PolymorphicFunction[..., object]] | None = None,
        experimental_debug_stripper: bool = False,
        experimental_io_device: str | None = None,
        experimental_variable_policy: str | VariablePolicy | None = None,
        experimental_custom_gradients: bool = True,
        experimental_image_format: bool = False,
        experimental_skip_saver: bool = False,
        experimental_sharding_callback=None,
        extra_tags=None,
    ) -> None: ...

def contains_saved_model(export_dir: str | Path) -> bool: ...

class _LoadedAttributes(Generic[_P, _R_co]):
    signatures: Mapping[str, ConcreteFunction[_P, _R_co]]

class _LoadedModel(AutoTrackable, _LoadedAttributes[_P, _R_co]):
    variables: list[tf.Variable]
    trainable_variables: list[tf.Variable]
    # TF1 model artifact specific
    graph: tf.Graph

def load(
    export_dir: str, tags: str | Sequence[str] | None = None, options: LoadOptions | None = None
) -> _LoadedModel[..., Any]: ...

_TF_Function: TypeAlias = ConcreteFunction[..., object] | PolymorphicFunction[..., object]

def save(
    obj: tf.Module,
    export_dir: str,
    signatures: _TF_Function | Mapping[str, _TF_Function] | None = None,
    options: SaveOptions | None = None,
) -> None: ...

ASSETS_DIRECTORY: str = "assets"
ASSETS_KEY: str = "saved_model_assets"
CLASSIFY_INPUTS: str = "inputs"
CLASSIFY_METHOD_NAME: str = "tensorflow/serving/classify"
CLASSIFY_OUTPUT_CLASSES: str = "classes"
CLASSIFY_OUTPUT_SCORES: str = "scores"
DEBUG_DIRECTORY: str = "debug"
DEBUG_INFO_FILENAME_PB: str = "saved_model_debug_info.pb"
DEFAULT_SERVING_SIGNATURE_DEF_KEY: str = "serving_default"
GPU: str = "gpu"
PREDICT_INPUTS: str = "inputs"
PREDICT_METHOD_NAME: str = "tensorflow/serving/predict"
PREDICT_OUTPUTS: str = "outputs"
REGRESS_INPUTS: str = "inputs"
REGRESS_METHOD_NAME: str = "tensorflow/serving/regress"
REGRESS_OUTPUTS: str = "outputs"
SAVED_MODEL_FILENAME_PB: str = "saved_model.pb"
SAVED_MODEL_FILENAME_PBTXT: str = "saved_model.pbtxt"
SAVED_MODEL_SCHEMA_VERSION: int = 1
SERVING: str = "serve"
TPU: str = "tpu"
TRAINING: str = "train"
VARIABLES_DIRECTORY: str = "variables"
VARIABLES_FILENAME: str = "variables"



from _typeshed import Incomplete

from kafka.sasl.abc import SaslMechanism

log: Incomplete

class SaslMechanismSSPI(SaslMechanism):
    SASL_QOP_AUTH: int
    SASL_QOP_AUTH_INT: int
    SASL_QOP_AUTH_CONF: int
    auth_id: Incomplete
    def __init__(self, **config) -> None: ...
    def auth_bytes(self): ...
    def receive(self, auth_bytes) -> None: ...
    def is_done(self): ...
    def is_authenticated(self): ...
    def auth_details(self): ...



from _typeshed import Incomplete

from kafka.protocol.api import Request, Response

class MetadataResponse_v0(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class MetadataResponse_v1(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class MetadataResponse_v2(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class MetadataResponse_v3(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class MetadataResponse_v4(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class MetadataResponse_v5(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class MetadataResponse_v6(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class MetadataResponse_v7(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class MetadataResponse_v8(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class MetadataRequest_v0(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = MetadataResponse_v0
    SCHEMA: Incomplete
    ALL_TOPICS: Incomplete
    NO_TOPICS: Incomplete

class MetadataRequest_v1(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = MetadataResponse_v1
    SCHEMA: Incomplete
    ALL_TOPICS: Incomplete
    NO_TOPICS: Incomplete

class MetadataRequest_v2(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = MetadataResponse_v2
    SCHEMA: Incomplete
    ALL_TOPICS: Incomplete
    NO_TOPICS: Incomplete

class MetadataRequest_v3(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = MetadataResponse_v3
    SCHEMA: Incomplete
    ALL_TOPICS: Incomplete
    NO_TOPICS: Incomplete

class MetadataRequest_v4(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = MetadataResponse_v4
    SCHEMA: Incomplete
    ALL_TOPICS: Incomplete
    NO_TOPICS: Incomplete

class MetadataRequest_v5(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = MetadataResponse_v5
    SCHEMA: Incomplete
    ALL_TOPICS: Incomplete
    NO_TOPICS: Incomplete

class MetadataRequest_v6(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = MetadataResponse_v6
    SCHEMA: Incomplete
    ALL_TOPICS: Incomplete
    NO_TOPICS: Incomplete

class MetadataRequest_v7(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = MetadataResponse_v7
    SCHEMA: Incomplete
    ALL_TOPICS: Incomplete
    NO_TOPICS: Incomplete

class MetadataRequest_v8(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = MetadataResponse_v8
    SCHEMA: Incomplete
    ALL_TOPICS: Incomplete
    NO_TOPICS: Incomplete

MetadataRequest: Incomplete
MetadataResponse: Incomplete



from _typeshed import Incomplete

from kafka.coordinator.base import BaseCoordinator

log: Incomplete

class ConsumerCoordinator(BaseCoordinator):
    DEFAULT_CONFIG: Incomplete
    config: Incomplete
    auto_commit_interval: Incomplete
    next_auto_commit_deadline: Incomplete
    completed_offset_commits: Incomplete
    def __init__(self, client, subscription, **configs) -> None: ...
    def __del__(self) -> None: ...
    def protocol_type(self): ...
    def group_protocols(self): ...
    def poll(self, timeout_ms=None): ...
    def time_to_next_poll(self): ...
    def need_rejoin(self): ...
    def refresh_committed_offsets_if_needed(self, timeout_ms=None): ...
    def fetch_committed_offsets(self, partitions, timeout_ms=None): ...
    def close(self, autocommit: bool = True, timeout_ms=None) -> None: ...  # type: ignore[override]
    def commit_offsets_async(self, offsets, callback=None): ...
    def commit_offsets_sync(self, offsets, timeout_ms=None): ...
    def maybe_auto_commit_offsets_now(self) -> None: ...

class ConsumerCoordinatorMetrics:
    metrics: Incomplete
    metric_group_name: Incomplete
    commit_latency: Incomplete
    def __init__(self, metrics, metric_group_prefix, subscription) -> None: ...



from typing import Final

esc: Final = "\x1b["
codes: Final[dict[str, str]]
dark_colors: Final[list[str]]
light_colors: Final[list[str]]

def reset_color() -> str: ...
def colorize(color_key: str, text: str) -> str: ...
def ansiformat(attr: str, text: str) -> str: ...



from _typeshed import Incomplete
from datetime import tzinfo

class OffsetTzInfo(tzinfo):
    offset: Incomplete
    name: Incomplete
    def __init__(self, offset, name) -> None: ...
    def utcoffset(self, dt): ...
    def tzname(self, dt): ...
    def dst(self, dt): ...
    def __getinitargs__(self): ...



from _typeshed import Incomplete

from kafka.protocol.struct import Struct

class ConsumerProtocolMemberMetadata_v0(Struct):
    SCHEMA: Incomplete

class ConsumerProtocolMemberAssignment_v0(Struct):
    SCHEMA: Incomplete
    def partitions(self): ...

class ConsumerProtocol_v0:
    PROTOCOL_TYPE: str
    METADATA = ConsumerProtocolMemberMetadata_v0
    ASSIGNMENT = ConsumerProtocolMemberAssignment_v0

ConsumerProtocol: Incomplete



from datetime import timedelta

from google.protobuf.duration_pb2 import Duration

def from_json_string(value: str) -> Duration: ...
def from_microseconds(micros: float) -> Duration: ...
def from_milliseconds(millis: float) -> Duration: ...
def from_nanoseconds(nanos: float) -> Duration: ...
def from_seconds(seconds: float) -> Duration: ...
def from_timedelta(td: timedelta) -> Duration: ...
def to_json_string(duration: Duration) -> str: ...
def to_microseconds(duration: Duration) -> int: ...
def to_milliseconds(duration: Duration) -> int: ...
def to_nanoseconds(duration: Duration) -> int: ...
def to_seconds(duration: Duration) -> int: ...
def to_timedelta(duration: Duration) -> timedelta: ...



import sys
from _typeshed import MaybeNone, OptExcInfo, ProfileFunction, StrOrBytesPath, TraceFunction, structseq
from _typeshed.importlib import MetaPathFinderProtocol, PathEntryFinderProtocol
from builtins import object as _object
from collections.abc import AsyncGenerator, Callable, Sequence
from io import TextIOWrapper
from types import FrameType, ModuleType, SimpleNamespace, TracebackType
from typing import Any, Final, Literal, NoReturn, Protocol, TextIO, TypeAlias, TypeVar, final, overload, type_check_only
from typing_extensions import LiteralString, deprecated

_T = TypeVar("_T")
_LazyImportMode: TypeAlias = Literal["normal", "all", "none"]
_LazyImportFilter: TypeAlias = Callable[[str | None, str, tuple[str, ...] | None], bool]

# see https://github.com/python/typeshed/issues/8513#issue-1333671093 for the rationale behind this alias
_ExitCode: TypeAlias = str | int | None

if sys.version_info >= (3, 15):
    @type_check_only
    class _AbiInfo(SimpleNamespace):
        pointer_bits: int
        free_threaded: bool
        debug: bool
        byteorder: Literal["little", "big"]

# ----- sys variables -----
if sys.platform != "win32":
    abiflags: str
if sys.version_info >= (3, 15):
    abi_info: _AbiInfo
argv: list[str]
base_exec_prefix: str
base_prefix: str
byteorder: Literal["little", "big"]
builtin_module_names: Sequence[str]  # actually a tuple of strings
copyright: str
if sys.platform == "win32":
    dllhandle: int
dont_write_bytecode: bool
displayhook: Callable[[object], Any]
excepthook: Callable[[type[BaseException], BaseException, TracebackType | None], Any]
exec_prefix: str
executable: str
float_repr_style: Literal["short", "legacy"]
hexversion: int
last_type: type[BaseException] | None
last_value: BaseException | None
last_traceback: TracebackType | None
if sys.version_info >= (3, 12):
    last_exc: BaseException  # or undefined.
maxsize: int
maxunicode: int
meta_path: list[MetaPathFinderProtocol]
modules: dict[str, ModuleType]
if sys.version_info >= (3, 15):
    lazy_modules: set[str]
orig_argv: list[str]
path: list[str]
path_hooks: list[Callable[[str], PathEntryFinderProtocol]]
path_importer_cache: dict[str, PathEntryFinderProtocol | None]
platform: LiteralString
platlibdir: str
prefix: str
pycache_prefix: str | None
ps1: object
ps2: object

# TextIO is used instead of more specific types for the standard streams,
# since they are often monkeypatched at runtime. At startup, the objects
# are initialized to instances of TextIOWrapper, but can also be None under
# some circumstances.
#
# To use methods from TextIOWrapper, use an isinstance check to ensure that
# the streams have not been overridden:
#
# if isinstance(sys.stdout, io.TextIOWrapper):
#    sys.stdout.reconfigure(...)
stdin: TextIO | MaybeNone
stdout: TextIO | MaybeNone
stderr: TextIO | MaybeNone
stdlib_module_names: frozenset[str]

__stdin__: Final[TextIOWrapper | None]  # Contains the original value of stdin
__stdout__: Final[TextIOWrapper | None]  # Contains the original value of stdout
__stderr__: Final[TextIOWrapper | None]  # Contains the original value of stderr
tracebacklimit: int | None
version: str
api_version: int
warnoptions: Any
#  Each entry is a tuple of the form (action, message, category, module,
#    lineno)
if sys.platform == "win32":
    winver: str
_xoptions: dict[Any, Any]

# Type alias used as a mixin for structseq classes that cannot be instantiated at runtime
# This can't be represented in the type system, so we just use `structseq[Any]`
_UninstantiableStructseq: TypeAlias = structseq[Any]

flags: _flags

# This class is not exposed at runtime. It calls itself sys.flags.
# As a tuple, it can have a length between 15 and 18. We don't model
# the exact length here because that varies by patch version due to
# the backported security fix int_max_str_digits. The exact length shouldn't
# be relied upon. See #13031
# This can be re-visited when typeshed drops support for 3.10,
# at which point all supported versions will include int_max_str_digits
# in all patch versions.
# 3.9 is 15 or 16-tuple
# 3.10 is 16 or 17-tuple
# 3.11+ is an 18-tuple.
@final
@type_check_only
class _flags(_UninstantiableStructseq, tuple[int, ...]):
    # `safe_path` was added in py311
    if sys.version_info >= (3, 11):
        __match_args__: Final = (
            "debug",
            "inspect",
            "interactive",
            "optimize",
            "dont_write_bytecode",
            "no_user_site",
            "no_site",
            "ignore_environment",
            "verbose",
            "bytes_warning",
            "quiet",
            "hash_randomization",
            "isolated",
            "dev_mode",
            "utf8_mode",
            "warn_default_encoding",
            "safe_path",
            "int_max_str_digits",
        )
    else:
        __match_args__: Final = (
            "debug",
            "inspect",
            "interactive",
            "optimize",
            "dont_write_bytecode",
            "no_user_site",
            "no_site",
            "ignore_environment",
            "verbose",
            "bytes_warning",
            "quiet",
            "hash_randomization",
            "isolated",
            "dev_mode",
            "utf8_mode",
            "warn_default_encoding",
            "int_max_str_digits",
        )

    @property
    def debug(self) -> int: ...
    @property
    def inspect(self) -> int: ...
    @property
    def interactive(self) -> int: ...
    @property
    def optimize(self) -> int: ...
    @property
    def dont_write_bytecode(self) -> int: ...
    @property
    def no_user_site(self) -> int: ...
    @property
    def no_site(self) -> int: ...
    @property
    def ignore_environment(self) -> int: ...
    @property
    def verbose(self) -> int: ...
    @property
    def bytes_warning(self) -> int: ...
    @property
    def quiet(self) -> int: ...
    @property
    def hash_randomization(self) -> int: ...
    @property
    def isolated(self) -> int: ...
    @property
    def dev_mode(self) -> bool: ...
    @property
    def utf8_mode(self) -> int: ...
    @property
    def warn_default_encoding(self) -> int: ...
    if sys.version_info >= (3, 11):
        @property
        def safe_path(self) -> bool: ...
    if sys.version_info >= (3, 13):
        @property
        def gil(self) -> Literal[0, 1]: ...
    if sys.version_info >= (3, 14):
        @property
        def thread_inherit_context(self) -> Literal[0, 1]: ...
        @property
        def context_aware_warnings(self) -> Literal[0, 1]: ...
    # Whether or not this exists on lower versions of Python
    # may depend on which patch release you're using
    # (it was backported to all Python versions on 3.8+ as a security fix)
    # Added in: 3.9.14, 3.10.7
    # and present in all versions of 3.11 and later.
    @property
    def int_max_str_digits(self) -> int: ...

float_info: _float_info

# This class is not exposed at runtime. It calls itself sys.float_info.
@final
@type_check_only
class _float_info(structseq[float], tuple[float, int, int, float, int, int, int, int, float, int, int]):
    __match_args__: Final = (
        "max",
        "max_exp",
        "max_10_exp",
        "min",
        "min_exp",
        "min_10_exp",
        "dig",
        "mant_dig",
        "epsilon",
        "radix",
        "rounds",
    )

    @property
    def max(self) -> float: ...  # DBL_MAX
    @property
    def max_exp(self) -> int: ...  # DBL_MAX_EXP
    @property
    def max_10_exp(self) -> int: ...  # DBL_MAX_10_EXP
    @property
    def min(self) -> float: ...  # DBL_MIN
    @property
    def min_exp(self) -> int: ...  # DBL_MIN_EXP
    @property
    def min_10_exp(self) -> int: ...  # DBL_MIN_10_EXP
    @property
    def dig(self) -> int: ...  # DBL_DIG
    @property
    def mant_dig(self) -> int: ...  # DBL_MANT_DIG
    @property
    def epsilon(self) -> float: ...  # DBL_EPSILON
    @property
    def radix(self) -> int: ...  # FLT_RADIX
    @property
    def rounds(self) -> int: ...  # FLT_ROUNDS

hash_info: _hash_info

# This class is not exposed at runtime. It calls itself sys.hash_info.
@final
@type_check_only
class _hash_info(structseq[Any | int], tuple[int, int, int, int, int, str, int, int, int]):
    __match_args__: Final = ("width", "modulus", "inf", "nan", "imag", "algorithm", "hash_bits", "seed_bits", "cutoff")

    @property
    def width(self) -> int: ...
    @property
    def modulus(self) -> int: ...
    @property
    def inf(self) -> int: ...
    @property
    def nan(self) -> int: ...
    @property
    def imag(self) -> int: ...
    @property
    def algorithm(self) -> str: ...
    @property
    def hash_bits(self) -> int: ...
    @property
    def seed_bits(self) -> int: ...
    @property
    def cutoff(self) -> int: ...  # undocumented

implementation: _implementation

# This class isn't really a thing. At runtime, implementation is an instance
# of types.SimpleNamespace. This allows for better typing.
@type_check_only
class _implementation:
    name: str
    version: _version_info
    hexversion: int
    cache_tag: str
    # Define __getattr__, as the documentation states:
    # > sys.implementation may contain additional attributes specific to the Python implementation.
    # > These non-standard attributes must start with an underscore, and are not described here.
    def __getattr__(self, name: str) -> Any: ...

int_info: _int_info

# This class is not exposed at runtime. It calls itself sys.int_info.
@final
@type_check_only
class _int_info(structseq[int], tuple[int, int, int, int]):
    __match_args__: Final = ("bits_per_digit", "sizeof_digit", "default_max_str_digits", "str_digits_check_threshold")

    @property
    def bits_per_digit(self) -> int: ...
    @property
    def sizeof_digit(self) -> int: ...
    @property
    def default_max_str_digits(self) -> int: ...
    @property
    def str_digits_check_threshold(self) -> int: ...

_ThreadInfoName: TypeAlias = Literal["nt", "pthread", "pthread-stubs", "solaris"]
_ThreadInfoLock: TypeAlias = Literal["semaphore", "mutex+cond"] | None

# This class is not exposed at runtime. It calls itself sys.thread_info.
@final
@type_check_only
class _thread_info(_UninstantiableStructseq, tuple[_ThreadInfoName, _ThreadInfoLock, str | None]):
    __match_args__: Final = ("name", "lock", "version")

    @property
    def name(self) -> _ThreadInfoName: ...
    @property
    def lock(self) -> _ThreadInfoLock: ...
    @property
    def version(self) -> str | None: ...

thread_info: _thread_info
_ReleaseLevel: TypeAlias = Literal["alpha", "beta", "candidate", "final"]

# This class is not exposed at runtime. It calls itself sys.version_info.
@final
@type_check_only
class _version_info(_UninstantiableStructseq, tuple[int, int, int, _ReleaseLevel, int]):
    __match_args__: Final = ("major", "minor", "micro", "releaselevel", "serial")

    @property
    def major(self) -> int: ...
    @property
    def minor(self) -> int: ...
    @property
    def micro(self) -> int: ...
    @property
    def releaselevel(self) -> _ReleaseLevel: ...
    @property
    def serial(self) -> int: ...

version_info: _version_info

def call_tracing(func: Callable[..., _T], args: Any, /) -> _T: ...

if sys.version_info >= (3, 13):
    @deprecated("Deprecated since Python 3.13. Use `_clear_internal_caches()` instead.")
    def _clear_type_cache() -> None: ...

else:
    def _clear_type_cache() -> None: ...

def _current_frames() -> dict[int, FrameType]: ...
def _getframe(depth: int = 0, /) -> FrameType: ...

# documented -- see https://docs.python.org/3/library/sys.html#sys._current_exceptions
if sys.version_info >= (3, 12):
    def _current_exceptions() -> dict[int, BaseException | None]: ...

else:
    def _current_exceptions() -> dict[int, OptExcInfo]: ...

if sys.version_info >= (3, 12):
    def _getframemodulename(depth: int = 0) -> str | None: ...

def _debugmallocstats() -> None: ...
def __displayhook__(object: object, /) -> None: ...
def __excepthook__(exctype: type[BaseException], value: BaseException, traceback: TracebackType | None, /) -> None: ...
def exc_info() -> OptExcInfo: ...

if sys.version_info >= (3, 11):
    def exception() -> BaseException | None: ...

def exit(status: _ExitCode = None, /) -> NoReturn: ...

if sys.platform == "android":  # noqa: Y008
    def getandroidapilevel() -> int: ...

def getallocatedblocks() -> int: ...
def getdefaultencoding() -> Literal["utf-8"]: ...

if sys.platform != "win32":
    def getdlopenflags() -> int: ...

def getfilesystemencoding() -> LiteralString: ...
def getfilesystemencodeerrors() -> LiteralString: ...

if sys.version_info >= (3, 15):
    def get_lazy_imports() -> _LazyImportMode: ...
    def get_lazy_imports_filter() -> _LazyImportFilter | None: ...

def getrefcount(object: Any, /) -> int: ...
def getrecursionlimit() -> int: ...
def getsizeof(obj: object, default: int = ...) -> int: ...
def getswitchinterval() -> float: ...
def getprofile() -> ProfileFunction | None: ...
def setprofile(function: ProfileFunction | None, /) -> None: ...
def gettrace() -> TraceFunction | None: ...
def settrace(function: TraceFunction | None, /) -> None: ...

if sys.platform == "win32":
    # A tuple of length 5, even though it has more than 5 attributes.
    @final
    @type_check_only
    class _WinVersion(_UninstantiableStructseq, tuple[int, int, int, int, str]):
        @property
        def major(self) -> int: ...
        @property
        def minor(self) -> int: ...
        @property
        def build(self) -> int: ...
        @property
        def platform(self) -> int: ...
        @property
        def service_pack(self) -> str: ...
        @property
        def service_pack_minor(self) -> int: ...
        @property
        def service_pack_major(self) -> int: ...
        @property
        def suite_mask(self) -> int: ...
        @property
        def product_type(self) -> int: ...
        @property
        def platform_version(self) -> tuple[int, int, int]: ...

    def getwindowsversion() -> _WinVersion: ...

@overload
def intern(string: LiteralString, /) -> LiteralString: ...
@overload
def intern(string: str, /) -> str: ...  # type: ignore[misc]

__interactivehook__: Callable[[], object]

if sys.version_info >= (3, 13):
    def _is_gil_enabled() -> bool: ...
    def _clear_internal_caches() -> None: ...
    def _is_interned(string: str, /) -> bool: ...

def is_finalizing() -> bool: ...
def breakpointhook(*args: Any, **kwargs: Any) -> Any: ...

__breakpointhook__ = breakpointhook  # Contains the original value of breakpointhook

if sys.platform != "win32":
    def setdlopenflags(flags: int, /) -> None: ...

def setrecursionlimit(limit: int, /) -> None: ...
def setswitchinterval(interval: float, /) -> None: ...
def gettotalrefcount() -> int: ...  # Debug builds only

# Doesn't exist at runtime, but exported in the stubs so pytest etc. can annotate their code more easily.
@type_check_only
class UnraisableHookArgs(Protocol):
    exc_type: type[BaseException]
    exc_value: BaseException | None
    exc_traceback: TracebackType | None
    err_msg: str | None
    object: _object

unraisablehook: Callable[[UnraisableHookArgs], Any]

def __unraisablehook__(unraisable: UnraisableHookArgs, /) -> Any: ...
def addaudithook(hook: Callable[[str, tuple[Any, ...]], Any]) -> None: ...
def audit(event: str, /, *args: Any) -> None: ...

_AsyncgenHook: TypeAlias = Callable[[AsyncGenerator[Any, Any]], None] | None

# This class is not exposed at runtime. It calls itself builtins.asyncgen_hooks.
@final
@type_check_only
class _asyncgen_hooks(structseq[_AsyncgenHook], tuple[_AsyncgenHook, _AsyncgenHook]):
    __match_args__: Final = ("firstiter", "finalizer")

    @property
    def firstiter(self) -> _AsyncgenHook: ...
    @property
    def finalizer(self) -> _AsyncgenHook: ...

def get_asyncgen_hooks() -> _asyncgen_hooks: ...
def set_asyncgen_hooks(firstiter: _AsyncgenHook = ..., finalizer: _AsyncgenHook = ...) -> None: ...

if sys.platform == "win32":
    if sys.version_info >= (3, 13):
        @deprecated(
            "Deprecated since Python 3.13; will be removed in Python 3.16. "
            "Use the `PYTHONLEGACYWINDOWSFSENCODING` environment variable instead."
        )
        def _enablelegacywindowsfsencoding() -> None: ...
    else:
        def _enablelegacywindowsfsencoding() -> None: ...

def get_coroutine_origin_tracking_depth() -> int: ...
def set_coroutine_origin_tracking_depth(depth: int) -> None: ...

# The following two functions were added in 3.11.0, 3.10.7, and 3.9.14,
# as part of the response to CVE-2020-10735
def set_int_max_str_digits(maxdigits: int) -> None: ...
def get_int_max_str_digits() -> int: ...

if sys.version_info >= (3, 15):
    def set_lazy_imports(mode: _LazyImportMode) -> None: ...
    def set_lazy_imports_filter(filter: _LazyImportFilter | None) -> None: ...

if sys.version_info >= (3, 12):
    if sys.version_info >= (3, 13):
        def getunicodeinternedsize(*, _only_immortal: bool = False) -> int: ...
    else:
        def getunicodeinternedsize() -> int: ...

    def deactivate_stack_trampoline() -> None: ...
    def is_stack_trampoline_active() -> bool: ...
    # It always exists, but raises on non-linux platforms:
    if sys.platform == "linux":
        def activate_stack_trampoline(backend: str, /) -> None: ...
    else:
        def activate_stack_trampoline(backend: str, /) -> NoReturn: ...

    from . import _monitoring

    monitoring = _monitoring

if sys.version_info >= (3, 14):
    def is_remote_debug_enabled() -> bool: ...
    def remote_exec(pid: int, script: StrOrBytesPath) -> None: ...
    def _is_immortal(op: object, /) -> bool: ...

    from . import __jit

    _jit = __jit



from collections.abc import Iterable

from google.protobuf.descriptor import Descriptor, EnumDescriptor, FileDescriptor, ServiceDescriptor
from google.protobuf.descriptor_pool import DescriptorPool
from google.protobuf.message import Message

class SymbolDatabase:
    def __init__(self, pool: DescriptorPool | None = None) -> None: ...
    def RegisterMessage(self, message: type[Message] | Message) -> type[Message] | Message: ...
    def RegisterMessageDescriptor(self, message_descriptor: Descriptor) -> None: ...
    def RegisterEnumDescriptor(self, enum_descriptor: EnumDescriptor) -> EnumDescriptor: ...
    def RegisterServiceDescriptor(self, service_descriptor: ServiceDescriptor) -> None: ...
    def RegisterFileDescriptor(self, file_descriptor: FileDescriptor) -> None: ...
    def GetSymbol(self, symbol: str) -> type[Message]: ...
    def GetMessages(self, files: Iterable[str]) -> dict[str, type[Message]]: ...

def Default() -> SymbolDatabase: ...



from collections.abc import Callable
from typing import Any, TypeAlias, TypeVar

import numpy as np

__all__ = ["resample", "resample_nu"]

# np.floating[Any] because precision is not important
_FloatArray = TypeVar("_FloatArray", bound=np.ndarray[tuple[int, ...], np.dtype[np.floating[Any]]])
_FilterType: TypeAlias = str | Callable[[int], np.ndarray[tuple[int], np.dtype[np.float64]]]

def resample(
    x: _FloatArray,
    sr_orig: float,
    sr_new: float,
    axis: int = -1,
    filter: _FilterType = "kaiser_best",
    parallel: bool = False,
    *,
    num_zeros: int = 64,
    precision: int = 9,
    rolloff: float = 0.945,
) -> _FloatArray: ...
def resample_nu(
    x: _FloatArray,
    sr_orig: float,
    t_out: _FloatArray,
    axis: int = -1,
    filter: _FilterType = "kaiser_best",
    parallel: bool = False,
    *,
    num_zeros: int = 64,
    precision: int = 9,
    rolloff: float = 0.945,
) -> _FloatArray: ...



from _typeshed import Incomplete
from collections.abc import Callable, Sequence
from typing import Any, ClassVar, Final, Literal, TypeAlias

from docutils import nodes, parsers
from docutils.parsers.rst.states import Inliner, RSTState, RSTStateMachine
from docutils.statemachine import StringList
from docutils.transforms import Transform
from docutils.utils import Reporter

__docformat__: Final = "reStructuredText"

class Parser(parsers.Parser):
    config_section_dependencies: ClassVar[tuple[str, ...]]
    initial_state: Literal["Body", "RFC2822Body"]
    state_classes: Sequence[type[RSTState]]
    inliner: Inliner | None
    statemachine: RSTStateMachine
    def __init__(self, rfc2822: bool = False, inliner: Inliner | None = None) -> None: ...
    def get_transforms(self) -> list[type[Transform]]: ...
    def parse(self, inputstring: str, document: nodes.document) -> None: ...

class DirectiveError(Exception):
    level: int
    msg: str
    def __init__(self, level: int, message: str) -> None: ...

class Directive:
    required_arguments: ClassVar[int]
    optional_arguments: ClassVar[int]
    final_argument_whitespace: ClassVar[bool]
    option_spec: ClassVar[dict[str, Callable[[str], Incomplete]] | None]
    has_content: ClassVar[bool]
    name: str
    arguments: list[str]
    options: dict[str, Incomplete]
    content: StringList
    lineno: int
    content_offset: int
    block_text: str
    state: RSTState
    state_machine: RSTStateMachine = ...
    reporter: Reporter
    def __init__(
        self,
        name: str,
        arguments: list[str],
        options: dict[str, Incomplete],
        content: StringList,
        lineno: int,
        content_offset: int,
        block_text: str,
        state: RSTState,
        state_machine: RSTStateMachine,
    ) -> None: ...
    def run(self) -> Sequence[nodes.Node]: ...
    def directive_error(self, level: int, message: str) -> DirectiveError: ...
    def debug(self, message: str) -> DirectiveError: ...
    def info(self, message: str) -> DirectiveError: ...
    def warning(self, message: str) -> DirectiveError: ...
    def error(self, message: str) -> DirectiveError: ...
    def severe(self, message: str) -> DirectiveError: ...
    def assert_has_content(self) -> None: ...
    def add_name(self, node: nodes.Node) -> None: ...

_DirectiveFn: TypeAlias = Callable[
    [str, list[str], dict[str, Any], StringList, int, int, str, RSTState, RSTStateMachine], Directive
]

def convert_directive_function(directive_fn: _DirectiveFn) -> type[Directive]: ...



from _typeshed import SupportsRichComparison
from collections import OrderedDict
from collections.abc import Callable
from typing import IO, Any, TypeAlias, TypeVar, overload

from simplejson.decoder import JSONDecoder as JSONDecoder
from simplejson.encoder import JSONEncoder as JSONEncoder, JSONEncoderForHTML as JSONEncoderForHTML
from simplejson.raw_json import RawJSON as RawJSON
from simplejson.scanner import JSONDecodeError as JSONDecodeError

_LoadsString: TypeAlias = str | bytes | bytearray
_T = TypeVar("_T")

@overload
def dumps(
    obj: Any,
    skipkeys: bool = False,
    ensure_ascii: bool = True,
    check_circular: bool = True,
    allow_nan: bool = False,
    *,
    cls: type[JSONEncoder],
    indent: str | int | None = None,
    separators: tuple[str, str] | None = None,
    encoding: str | None = "utf-8",
    default: Callable[[Any], Any] | None = None,
    use_decimal: bool = True,
    namedtuple_as_object: bool = True,
    tuple_as_array: bool = True,
    bigint_as_string: bool = False,
    sort_keys: bool = False,
    item_sort_key: Callable[[Any], SupportsRichComparison] | None = None,
    for_json: bool = False,
    ignore_nan: bool = False,
    int_as_string_bitcount: int | None = None,
    iterable_as_array: bool = False,
    **kw: Any,
) -> str: ...
@overload
def dumps(
    obj: Any,
    skipkeys: bool = False,
    ensure_ascii: bool = True,
    check_circular: bool = True,
    allow_nan: bool = False,
    cls: type[JSONEncoder] | None = None,
    indent: str | int | None = None,
    separators: tuple[str, str] | None = None,
    encoding: str | None = "utf-8",
    default: Callable[[Any], Any] | None = None,
    use_decimal: bool = True,
    namedtuple_as_object: bool = True,
    tuple_as_array: bool = True,
    bigint_as_string: bool = False,
    sort_keys: bool = False,
    item_sort_key: Callable[[Any], SupportsRichComparison] | None = None,
    for_json: bool = False,
    ignore_nan: bool = False,
    int_as_string_bitcount: int | None = None,
    iterable_as_array: bool = False,
) -> str: ...

@overload
def dump(
    obj: Any,
    fp: IO[str],
    skipkeys: bool = False,
    ensure_ascii: bool = True,
    check_circular: bool = True,
    allow_nan: bool = False,
    *,
    cls: type[JSONEncoder],
    indent: str | int | None = None,
    separators: tuple[str, str] | None = None,
    encoding: str | None = "utf-8",
    default: Callable[[Any], Any] | None = None,
    use_decimal: bool = True,
    namedtuple_as_object: bool = True,
    tuple_as_array: bool = True,
    bigint_as_string: bool = False,
    sort_keys: bool = False,
    item_sort_key: Callable[[Any], SupportsRichComparison] | None = None,
    for_json: bool = False,
    ignore_nan: bool = False,
    int_as_string_bitcount: int | None = None,
    iterable_as_array: bool = False,
    **kw: Any,
) -> None: ...
@overload
def dump(
    obj: Any,
    fp: IO[str],
    skipkeys: bool = False,
    ensure_ascii: bool = True,
    check_circular: bool = True,
    allow_nan: bool = False,
    cls: type[JSONEncoder] | None = None,
    indent: str | int | None = None,
    separators: tuple[str, str] | None = None,
    encoding: str | None = "utf-8",
    default: Callable[[Any], Any] | None = None,
    use_decimal: bool = True,
    namedtuple_as_object: bool = True,
    tuple_as_array: bool = True,
    bigint_as_string: bool = False,
    sort_keys: bool = False,
    item_sort_key: Callable[[Any], SupportsRichComparison] | None = None,
    for_json: bool = False,
    ignore_nan: bool = False,
    int_as_string_bitcount: int | None = None,
    iterable_as_array: bool = False,
) -> None: ...

@overload
def loads(
    s: _LoadsString,
    encoding: str | None = None,
    *,
    cls: type[JSONDecoder],
    object_hook: Callable[[dict[Any, Any]], Any] | None = None,
    parse_float: Callable[[str], Any] | None = None,
    parse_int: Callable[[str], Any] | None = None,
    parse_constant: Callable[[str], Any] | None = None,
    object_pairs_hook: Callable[[list[tuple[Any, Any]]], Any] | None = None,
    use_decimal: bool = False,
    allow_nan: bool = False,
    **kw: Any,
) -> Any: ...
@overload
def loads(
    s: _LoadsString,
    encoding: str | None = None,
    cls: type[JSONDecoder] | None = None,
    object_hook: Callable[[dict[Any, Any]], Any] | None = None,
    parse_float: Callable[[str], Any] | None = None,
    parse_int: Callable[[str], Any] | None = None,
    parse_constant: Callable[[str], Any] | None = None,
    object_pairs_hook: Callable[[list[tuple[Any, Any]]], Any] | None = None,
    use_decimal: bool = False,
    allow_nan: bool = False,
) -> Any: ...

@overload
def load(
    fp: IO[str],
    encoding: str | None = None,
    *,
    cls: type[JSONDecoder],
    object_hook: Callable[[dict[Any, Any]], Any] | None = None,
    parse_float: Callable[[str], Any] | None = None,
    parse_int: Callable[[str], Any] | None = None,
    parse_constant: Callable[[str], Any] | None = None,
    object_pairs_hook: Callable[[list[tuple[Any, Any]]], Any] | None = None,
    use_decimal: bool = False,
    allow_nan: bool = False,
    **kw: Any,
) -> Any: ...
@overload
def load(
    fp: IO[str],
    encoding: str | None = None,
    cls: type[JSONDecoder] | None = None,
    object_hook: Callable[[dict[Any, Any]], Any] | None = None,
    parse_float: Callable[[str], Any] | None = None,
    parse_int: Callable[[str], Any] | None = None,
    parse_constant: Callable[[str], Any] | None = None,
    object_pairs_hook: Callable[[list[tuple[Any, Any]]], Any] | None = None,
    use_decimal: bool = False,
    allow_nan: bool = False,
) -> Any: ...

def simple_first(kv: tuple[_T, object]) -> tuple[bool, _T]: ...

__all__ = [
    "dump",
    "dumps",
    "load",
    "loads",
    "JSONDecoder",
    "JSONDecodeError",
    "JSONEncoder",
    "OrderedDict",
    "simple_first",
    "RawJSON",
]



from _typeshed import Incomplete
from collections.abc import Callable
from typing import ParamSpec, TypeVar

_P = ParamSpec("_P")
_T = TypeVar("_T")

MAX_INT: Incomplete
TO_SIGNED: Incomplete

def crc32(data): ...

class Timer:
    def __init__(self, timeout_ms, error_message=None, start_at=None) -> None: ...
    @property
    def expired(self): ...
    @property
    def timeout_ms(self): ...
    @property
    def elapsed_ms(self): ...
    def maybe_raise(self) -> None: ...

TOPIC_MAX_LENGTH: int
TOPIC_LEGAL_CHARS: Incomplete

def ensure_valid_topic_name(topic) -> None: ...

class WeakMethod:
    target: Incomplete
    method: Incomplete
    def __init__(self, object_dot_method) -> None: ...
    def __call__(self, *args, **kwargs): ...
    def __hash__(self): ...
    def __eq__(self, other): ...

class Dict(dict[Incomplete, Incomplete]): ...

def synchronized(func: Callable[_P, _T]) -> Callable[_P, _T]: ...



from _typeshed import Incomplete
from collections import deque
from logging import Logger

log: Logger

class KafkaProtocol:
    in_flight_requests: deque[tuple[int, Incomplete]]
    bytes_to_send: list[bytes]
    def __init__(
        self, client_id: str | None = None, api_version: tuple[int, int, int] | None = None, max_frame_size: int = 100000000
    ) -> None: ...
    def send_request(self, request, correlation_id: int | None = None) -> int: ...
    def send_bytes(self) -> bytes: ...
    def receive_bytes(self, data: bytes) -> list[Incomplete]: ...



from collections.abc import Iterable

def lookup_mux_register(
    gpio_pin: int, pin_defs: Iterable[tuple[int, str, str, int, int, str, str, str | None, int | None, int]]
) -> int: ...
def main() -> None: ...



import datetime
import re
from _typeshed import Incomplete
from collections import OrderedDict
from collections.abc import Iterable, Sequence, Set as AbstractSet
from typing import Any, Final, TypedDict, type_check_only

from dateparser.conf import Settings
from dateparser.date import DateData, DateDataParser, _DetectLanguagesFunction
from dateparser.languages.loader import LocaleDataLoader
from dateparser.languages.locale import Locale
from dateparser.search.text_detection import FullTextLanguageDetector

@type_check_only
class _SearchDates(TypedDict):
    Language: str | None
    Dates: list[tuple[str, datetime.datetime]] | None

RELATIVE_REG: Final[re.Pattern[str]]

def date_is_relative(translation: str) -> bool: ...

class _ExactLanguageSearch:
    loader: LocaleDataLoader
    language: Locale | None
    def __init__(self, loader: LocaleDataLoader) -> None: ...
    def get_current_language(self, shortname: str) -> None: ...
    def search(self, shortname: str, text: str, settings: Settings | None) -> tuple[list[str], list[str]]: ...
    @staticmethod
    def set_relative_base(
        substring: str, already_parsed: list[tuple[DateData, bool]]
    ) -> tuple[str, datetime.datetime | None]: ...
    def choose_best_split(
        self, possible_parsed_splits: list[Incomplete], possible_substrings_splits: list[Incomplete]
    ) -> tuple[Incomplete, Incomplete]: ...
    def split_by(self, item: str, original: str, splitter: str) -> list[list[list[str]]]: ...
    def split_if_not_parsed(self, item: str, original: str) -> list[list[list[str]]]: ...
    def parse_item(
        self,
        parser: DateDataParser,
        item: str,
        translated_item: str,
        parsed: list[tuple[DateData, bool]],
        need_relative_base: bool | None,
    ) -> tuple[DateData, bool]: ...
    def parse_found_objects(
        self,
        parser: DateDataParser,
        to_parse: Iterable[str],
        original: Sequence[str],
        translated: Sequence[str],
        settings: Settings,
    ) -> tuple[list[tuple[DateData, bool]], list[str]]: ...
    def search_parse(self, shortname: str, text: str, settings: Settings) -> list[tuple[str, datetime.datetime]]: ...

class DateSearchWithDetection:
    loader: LocaleDataLoader
    available_language_map: OrderedDict[str, Locale]
    search: _ExactLanguageSearch
    language_detector: FullTextLanguageDetector
    def __init__(self) -> None: ...
    def detect_language(
        self,
        text: str,
        languages: list[str] | tuple[str, ...] | AbstractSet[str] | None,
        settings: Settings | dict[str, Any] | None = None,
        detect_languages_function: _DetectLanguagesFunction | None = None,
    ) -> str | None: ...
    def search_dates(
        self,
        text: str,
        languages: list[str] | tuple[str, ...] | AbstractSet[str] | None = None,
        settings: Settings | dict[str, Any] | None = None,
        detect_languages_function: _DetectLanguagesFunction | None = None,
    ) -> _SearchDates: ...
    def preprocess_text(self, text: str, languages: Iterable[str] | None) -> str: ...



from enum import IntEnum

class ResourceType(IntEnum):
    UNKNOWN = 0
    ANY = 1
    CLUSTER = 4
    DELEGATION_TOKEN = 6
    GROUP = 3
    TOPIC = 2
    TRANSACTIONAL_ID = 5

class ACLOperation(IntEnum):
    UNKNOWN = 0
    ANY = 1
    ALL = 2
    READ = 3
    WRITE = 4
    CREATE = 5
    DELETE = 6
    ALTER = 7
    DESCRIBE = 8
    CLUSTER_ACTION = 9
    DESCRIBE_CONFIGS = 10
    ALTER_CONFIGS = 11
    IDEMPOTENT_WRITE = 12
    CREATE_TOKENS = 13
    DESCRIBE_TOKENS = 13

class ACLPermissionType(IntEnum):
    UNKNOWN = 0
    ANY = 1
    DENY = 2
    ALLOW = 3

class ACLResourcePatternType(IntEnum):
    UNKNOWN = 0
    ANY = 1
    MATCH = 2
    LITERAL = 3
    PREFIXED = 4

class ACLFilter:
    principal: str | None
    host: str | None
    operation: ACLOperation
    permission_type: ACLPermissionType
    resource_pattern: ResourcePatternFilter
    def __init__(
        self,
        principal: str | None,
        host: str | None,
        operation: ACLOperation,
        permission_type: ACLPermissionType,
        resource_pattern: ResourcePatternFilter,
    ) -> None: ...
    def validate(self) -> None: ...
    def __eq__(self, other): ...
    def __hash__(self): ...

class ACL(ACLFilter):
    resource_pattern: ResourcePattern
    def __init__(
        self,
        principal: str,
        host: str,
        operation: ACLOperation,
        permission_type: ACLPermissionType,
        resource_pattern: ResourcePattern,
    ) -> None: ...
    def validate(self) -> None: ...

class ResourcePatternFilter:
    resource_type: ResourceType
    resource_name: str | None
    pattern_type: ACLResourcePatternType
    def __init__(self, resource_type: ResourceType, resource_name: str | None, pattern_type: ACLResourcePatternType) -> None: ...
    def validate(self) -> None: ...
    def __eq__(self, other): ...
    def __hash__(self): ...

class ResourcePattern(ResourcePatternFilter):
    resource_name: str
    def __init__(
        self,
        resource_type: ResourceType,
        resource_name: str,
        pattern_type: ACLResourcePatternType = ACLResourcePatternType.LITERAL,
    ) -> None: ...
    def validate(self) -> None: ...

def valid_acl_operations(int_vals) -> set[ACLOperation]: ...



from _typeshed import Incomplete

from kafka.metrics.metrics_reporter import AbstractMetricsReporter

logger: Incomplete

class DictReporter(AbstractMetricsReporter):
    def __init__(self, prefix: str = "") -> None: ...
    def snapshot(self): ...
    def init(self, metrics) -> None: ...
    def metric_change(self, metric) -> None: ...
    def metric_removal(self, metric): ...
    def get_category(self, metric): ...
    def configure(self, configs) -> None: ...
    def close(self) -> None: ...



from _typeshed import Incomplete
from typing import NamedTuple

class TopicPartition(NamedTuple):
    topic: str
    partition: int

class BrokerMetadata(NamedTuple):
    nodeId: int
    host: str
    port: int
    rack: str | None

class PartitionMetadata(NamedTuple):
    topic: str
    partition: int
    leader: int
    leader_epoch: int | None
    replicas: list[int]
    isr: list[int]
    offline_replicas: list[int]
    error: Incomplete

class OffsetAndMetadata(NamedTuple):
    offset: int
    metadata: str
    leader_epoch: int

class OffsetAndTimestamp(NamedTuple):
    offset: int
    timestamp: int
    leader_epoch: int

class MemberInformation(NamedTuple):
    member_id: str
    client_id: str
    client_host: str
    member_metadata: Incomplete
    member_assignment: Incomplete

class GroupInformation(NamedTuple):
    error_code: int
    group: str
    state: str
    protocol_type: str
    protocol: str
    members: list[MemberInformation]
    authorized_operations: list[str]

class RetryOptions(NamedTuple):
    limit: int
    backoff_ms: int
    retry_on_timeouts: bool



class KafkaMetric:
    def __init__(self, metric_name, measurable, config) -> None: ...
    @property
    def metric_name(self): ...
    @property
    def measurable(self): ...

    @property
    def config(self): ...
    @config.setter
    def config(self, config) -> None: ...

    def value(self, time_ms=None): ...



from _typeshed import Incomplete
from collections.abc import Callable
from typing import Any, Concatenate, ParamSpec, TypeVar

from behave.runner import Context

_T = TypeVar("_T")
_F = TypeVar("_F", bound=Callable[..., Any])
_P = ParamSpec("_P")

def use_fixture(
    fixture_func: Callable[Concatenate[Context, _P], _T], context: Context, *fixture_args: _P.args, **fixture_kwargs: _P.kwargs
) -> _T: ...
def fixture(func: _F | None = None, name: str | None = None, pattern: str | None = None) -> _F: ...
def __getattr__(name: str) -> Incomplete: ...



from _typeshed import Incomplete
from collections import OrderedDict as OrderedDict

log: Incomplete
socketpair: Incomplete

class KafkaClient:
    DEFAULT_CONFIG: Incomplete
    config: Incomplete
    cluster: Incomplete
    def __init__(self, **configs) -> None: ...
    def maybe_connect(self, node_id, wakeup: bool = True): ...
    def connection_failed(self, node_id): ...
    def ready(self, node_id, metadata_priority: bool = True): ...
    def connected(self, node_id): ...
    def close(self, node_id=None) -> None: ...
    def __del__(self) -> None: ...
    def is_disconnected(self, node_id): ...
    def connection_delay(self, node_id): ...
    def throttle_delay(self, node_id): ...
    def is_ready(self, node_id, metadata_priority: bool = True): ...
    def send(self, node_id, request, wakeup: bool = True, request_timeout_ms=None): ...
    def poll(self, timeout_ms=None, future=None): ...
    def in_flight_request_count(self, node_id=None): ...
    def least_loaded_node(self): ...
    def least_loaded_node_refresh_ms(self): ...
    def set_topics(self, topics): ...
    def add_topic(self, topic): ...
    def get_api_versions(self): ...
    def check_version(self, node_id=None, timeout=None, **kwargs): ...
    def api_version(self, operation, max_version=None): ...
    def wakeup(self) -> None: ...
    def bootstrap_connected(self): ...
    def await_ready(self, node_id, timeout_ms: int = 30000): ...
    def send_and_receive(self, node_id, request): ...

class IdleConnectionManager:
    connections_max_idle: Incomplete
    next_idle_close_check_time: Incomplete
    lru_connections: Incomplete
    def __init__(self, connections_max_idle_ms) -> None: ...
    def update(self, conn_id) -> None: ...
    def remove(self, conn_id) -> None: ...
    def is_expired(self, conn_id): ...
    def next_check_ms(self): ...
    def update_next_idle_close_check_time(self, ts) -> None: ...
    def poll_expired_connection(self): ...

class KafkaClientMetrics:
    metrics: Incomplete
    metric_group_name: Incomplete
    connection_closed: Incomplete
    connection_created: Incomplete
    select_time: Incomplete
    io_time: Incomplete
    def __init__(self, metrics, metric_group_prefix, conns) -> None: ...



from _typeshed import Incomplete

from kafka.protocol.api import Request, Response

class OffsetCommitResponse_v0(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class OffsetCommitResponse_v1(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class OffsetCommitResponse_v2(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class OffsetCommitResponse_v3(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class OffsetCommitResponse_v4(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class OffsetCommitResponse_v5(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class OffsetCommitResponse_v6(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class OffsetCommitResponse_v7(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class OffsetCommitRequest_v0(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = OffsetCommitResponse_v0
    SCHEMA: Incomplete

class OffsetCommitRequest_v1(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = OffsetCommitResponse_v1
    SCHEMA: Incomplete

class OffsetCommitRequest_v2(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = OffsetCommitResponse_v2
    SCHEMA: Incomplete
    DEFAULT_RETENTION_TIME: int

class OffsetCommitRequest_v3(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = OffsetCommitResponse_v3
    SCHEMA: Incomplete
    DEFAULT_RETENTION_TIME: int

class OffsetCommitRequest_v4(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = OffsetCommitResponse_v4
    SCHEMA: Incomplete
    DEFAULT_RETENTION_TIME: int

class OffsetCommitRequest_v5(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = OffsetCommitResponse_v5
    SCHEMA: Incomplete

class OffsetCommitRequest_v6(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = OffsetCommitResponse_v6
    SCHEMA: Incomplete

class OffsetCommitRequest_v7(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = OffsetCommitResponse_v7
    SCHEMA: Incomplete

OffsetCommitRequest: Incomplete
OffsetCommitResponse: Incomplete

class OffsetFetchResponse_v0(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class OffsetFetchResponse_v1(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class OffsetFetchResponse_v2(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class OffsetFetchResponse_v3(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class OffsetFetchResponse_v4(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class OffsetFetchResponse_v5(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class OffsetFetchRequest_v0(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = OffsetFetchResponse_v0
    SCHEMA: Incomplete

class OffsetFetchRequest_v1(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = OffsetFetchResponse_v1
    SCHEMA: Incomplete

class OffsetFetchRequest_v2(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = OffsetFetchResponse_v2
    SCHEMA: Incomplete

class OffsetFetchRequest_v3(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = OffsetFetchResponse_v3
    SCHEMA: Incomplete

class OffsetFetchRequest_v4(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = OffsetFetchResponse_v4
    SCHEMA: Incomplete

class OffsetFetchRequest_v5(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = OffsetFetchResponse_v5
    SCHEMA: Incomplete

OffsetFetchRequest: Incomplete
OffsetFetchResponse: Incomplete



# Stub-only module, can't be imported at runtime.

from collections.abc import Collection
from typing import Any, Protocol, TypeAlias, type_check_only
from typing_extensions import TypeVar

import numpy as np

_ScalarT = TypeVar("_ScalarT", bound=bool | int | float | complex | str | bytes | np.generic)
_GenericT = TypeVar("_GenericT", bound=np.generic)
_GenericT_co = TypeVar("_GenericT_co", bound=np.generic, covariant=True)
_ShapeT_co = TypeVar("_ShapeT_co", bound=tuple[int, ...], default=Any, covariant=True)

# numpy aliases
@type_check_only
class SupportsArray(Protocol[_GenericT_co, _ShapeT_co]):
    def __array__(self) -> np.ndarray[_ShapeT_co, np.dtype[_GenericT_co]]: ...

ArrayLike1D: TypeAlias = Collection[_ScalarT] | SupportsArray[_GenericT, tuple[int]]
Array1D: TypeAlias = np.ndarray[tuple[int], np.dtype[_GenericT]]
Array2D: TypeAlias = np.ndarray[tuple[int, int], np.dtype[_GenericT]]
Seed: TypeAlias = int | np.random.Generator | np.random.RandomState



from webencodings import Encoding

from .ast import Node

def decode_stylesheet_bytes(
    css_bytes: bytes, protocol_encoding: str | None = None, environment_encoding: Encoding | None = None
) -> tuple[str, Encoding]: ...
def parse_stylesheet_bytes(
    css_bytes: bytes,
    protocol_encoding: str | None = None,
    environment_encoding: Encoding | None = None,
    skip_comments: bool = False,
    skip_whitespace: bool = False,
) -> tuple[list[Node], Encoding]: ...



import _typeshed
from collections import OrderedDict
from collections.abc import Iterator, Sequence
from functools import partial
from logging import Logger
from typing import Any, ClassVar, Generic, Literal, NoReturn, TypeAlias, TypeVar, overload
from typing_extensions import deprecated

from django.db.models import Field as DjangoField, Model, QuerySet
from django.utils.safestring import SafeString

from .declarative import DeclarativeMetaclass, ModelDeclarativeMetaclass
from .fields import Field
from .instance_loaders import BaseInstanceLoader
from .options import ResourceOptions
from .results import Error, Result, RowResult
from .widgets import ForeignKeyWidget, ManyToManyWidget, Widget

Dataset: TypeAlias = _typeshed.Incomplete  # tablib.Dataset
logger: Logger

def has_natural_foreign_key(model: Model) -> bool: ...

class Diff:
    left: list[str]
    right: list[str]
    new: bool
    def __init__(self, resource: Resource[_ModelT], instance: _ModelT, new: bool) -> None: ...
    def compare_with(self, resource: Resource[_ModelT], instance: _ModelT, dry_run: bool = False) -> None: ...
    def as_html(self) -> list[SafeString]: ...

_ModelT = TypeVar("_ModelT", bound=Model)

class Resource(Generic[_ModelT], metaclass=DeclarativeMetaclass):
    _meta: ResourceOptions[_ModelT]
    fields: OrderedDict[str, Field]
    create_instances: list[_ModelT]
    update_instances: list[_ModelT]
    delete_instances: list[_ModelT]
    def __init__(self, **kwargs: Any) -> None: ...
    @classmethod
    def get_result_class(self) -> type[Result]: ...
    @classmethod
    def get_row_result_class(self) -> type[RowResult]: ...
    @classmethod
    def get_error_result_class(self) -> type[Error]: ...
    @classmethod
    def get_diff_class(self) -> type[Diff]: ...
    @classmethod
    def get_db_connection_name(self) -> str: ...
    def get_use_transactions(self) -> bool: ...
    def get_chunk_size(self) -> int: ...
    @deprecated("The 'get_fields()' method is deprecated and will be removed in a future release.")
    def get_fields(self, **kwargs: Any) -> list[Field]: ...
    def get_field_name(self, field: Field) -> str: ...
    def init_instance(self, row: dict[str, Any] | None = None) -> _ModelT: ...
    def get_instance(self, instance_loader: BaseInstanceLoader, row: dict[str, Any]) -> _ModelT | None: ...
    def get_or_init_instance(self, instance_loader: BaseInstanceLoader, row: dict[str, Any]) -> tuple[_ModelT | None, bool]: ...
    def get_import_id_fields(self) -> Sequence[str]: ...
    def get_bulk_update_fields(self) -> list[str]: ...
    def bulk_create(
        self,
        using_transactions: bool,
        dry_run: bool,
        raise_errors: bool,
        batch_size: int | None = None,
        result: Result | None = None,
    ) -> None: ...
    def bulk_update(
        self,
        using_transactions: bool,
        dry_run: bool,
        raise_errors: bool,
        batch_size: int | None = None,
        result: Result | None = None,
    ) -> None: ...
    def bulk_delete(self, using_transactions: bool, dry_run: bool, raise_errors: bool, result: Result | None = None) -> None: ...
    def validate_instance(
        self, instance: _ModelT, import_validation_errors: dict[str, Any] | None = None, validate_unique: bool = True
    ) -> None: ...
    # For all the definitions below (from `save_instance()` to `import_row()`), `**kwargs` should contain:
    # dry_run: bool, use_transactions: bool, row_number: int, retain_instance_in_row_result: bool.
    # Users are free to pass extra arguments in `import_data()`so PEP 728 can probably be leveraged here.
    def save_instance(self, instance: _ModelT, is_create: bool, row: dict[str, Any], **kwargs: Any) -> None: ...
    def do_instance_save(self, instance: _ModelT) -> None: ...
    def before_save_instance(self, instance: _ModelT, row: dict[str, Any], **kwargs: Any) -> None: ...
    def after_save_instance(self, instance: _ModelT, row: dict[str, Any], **kwargs: Any) -> None: ...
    def delete_instance(self, instance: _ModelT, row: dict[str, Any], **kwargs: Any) -> None: ...
    def before_delete_instance(self, instance: _ModelT, row: dict[str, Any], **kwargs: Any) -> None: ...
    def after_delete_instance(self, instance: _ModelT, row: dict[str, Any], **kwargs: Any) -> None: ...
    def import_field(self, field: Field, instance: _ModelT, row: dict[str, Any], is_m2m: bool = False, **kwargs: Any) -> None: ...
    def get_import_fields(self) -> list[Field]: ...
    def import_instance(self, instance: _ModelT, row: dict[str, Any], **kwargs: Any) -> None: ...
    def save_m2m(self, instance: _ModelT, row: dict[str, Any], **kwargs: Any) -> None: ...
    def for_delete(self, row: dict[str, Any], instance: _ModelT) -> bool: ...
    def skip_row(
        self, instance: _ModelT, original: _ModelT, row: dict[str, Any], import_validation_errors: dict[str, Any] | None = None
    ) -> bool: ...
    def get_diff_headers(self) -> list[str]: ...
    def before_import(self, dataset: Dataset, **kwargs: Any) -> None: ...
    def after_import(self, dataset: Dataset, result: Result, **kwargs: Any) -> None: ...
    def before_import_row(self, row: dict[str, Any], **kwargs: Any) -> None: ...
    def after_import_row(self, row: dict[str, Any], row_result: RowResult, **kwargs: Any) -> None: ...
    def after_init_instance(self, instance: _ModelT, new: bool, row: dict[str, Any], **kwargs: Any) -> None: ...

    @overload
    def handle_import_error(self, result: Result, error: Exception, raise_errors: Literal[True]) -> NoReturn: ...
    @overload
    def handle_import_error(self, result: Result, error: Exception, raise_errors: Literal[False] = False) -> None: ...

    def import_row(self, row: dict[str, Any], instance_loader: BaseInstanceLoader, **kwargs: Any) -> RowResult: ...
    def import_data(
        self,
        dataset: Dataset,
        dry_run: bool = False,
        raise_errors: bool = False,
        use_transactions: bool | None = None,
        collect_failed_rows: bool = False,
        rollback_on_validation_errors: bool = False,
        **kwargs: Any,
    ) -> Result: ...
    def import_data_inner(
        self,
        dataset: Dataset,
        dry_run: bool,
        raise_errors: bool,
        using_transactions: bool,
        collect_failed_rows: bool,
        **kwargs: Any,
    ) -> Result: ...
    def get_import_order(self) -> tuple[str, ...]: ...
    def get_export_order(self) -> tuple[str, ...]: ...
    def before_export(self, queryset: QuerySet[_ModelT], **kwargs: Any) -> None: ...
    def after_export(self, queryset: QuerySet[_ModelT], dataset: Dataset, **kwargs: Any) -> None: ...
    def filter_export(self, queryset: QuerySet[_ModelT], **kwargs: Any) -> QuerySet[_ModelT]: ...
    def export_field(self, field: Field, instance: _ModelT, **kwargs: Any) -> str: ...
    def get_export_fields(self, selected_fields: Sequence[str] | None = None) -> list[Field]: ...
    def export_resource(self, instance: _ModelT, selected_fields: Sequence[str] | None = None, **kwargs: Any) -> list[str]: ...
    def get_export_headers(self, selected_fields: Sequence[str] | None = None) -> list[str]: ...
    def get_user_visible_headers(self) -> list[str]: ...
    def get_user_visible_fields(self) -> list[str]: ...
    def iter_queryset(self, queryset: QuerySet[_ModelT]) -> Iterator[_ModelT]: ...
    def export(self, queryset: QuerySet[_ModelT] | None = None, **kwargs: Any) -> Dataset: ...

class ModelResource(Resource[_ModelT], metaclass=ModelDeclarativeMetaclass):
    DEFAULT_RESOURCE_FIELD: ClassVar[type[Field]] = ...
    WIDGETS_MAP: ClassVar[dict[str, type[Widget]]]
    @classmethod
    def get_m2m_widget(cls, field: DjangoField[Any, Any]) -> partial[ManyToManyWidget[Any]]: ...
    @classmethod
    def get_fk_widget(cls, field: DjangoField[Any, Any]) -> partial[ForeignKeyWidget[Any]]: ...
    @classmethod
    def widget_from_django_field(cls, f: DjangoField[Any, Any], default: type[Widget] = ...) -> type[Widget]: ...
    @classmethod
    def widget_kwargs_for_field(cls, field_name: str, django_field: DjangoField[Any, Any]) -> dict[str, Any]: ...
    @classmethod
    def field_from_django_field(cls, field_name: str, django_field: DjangoField[Any, Any], readonly: bool) -> Field: ...
    def get_queryset(self) -> QuerySet[_ModelT]: ...
    def init_instance(self, row: dict[str, Any] | None = None) -> _ModelT: ...
    def after_import(self, dataset: Dataset, result: Result, **kwargs: Any) -> None: ...
    @classmethod
    def get_display_name(cls) -> str: ...

_ResourceT = TypeVar("_ResourceT", bound=Resource[Any])

# HK Type Vars could help type the first overload:
@overload
def modelresource_factory(model: Model, resource_class: type[_ResourceT]) -> _ResourceT: ...
@overload
def modelresource_factory(model: _ModelT) -> ModelResource[_ModelT]: ...



import selectors
import ssl
from _typeshed import Incomplete
from collections.abc import Callable, Mapping, Sequence
from typing import Literal, TypeAlias, TypedDict, type_check_only
from typing_extensions import Unpack

from kafka.producer.future import FutureRecordMetadata
from kafka.serializer.abstract import Serializer
from kafka.structs import OffsetAndMetadata, TopicPartition

_ApiVersion: TypeAlias = tuple[int, ...]
_BootstrapServers: TypeAlias = str | Sequence[str]
_KafkaClientFactory: TypeAlias = Callable[..., object]
_Partitioner: TypeAlias = Callable[[bytes | None, Sequence[int], Sequence[int]], int]
_ProducerSerializer: TypeAlias = Serializer | Callable[[object], bytes]
_SaslMechanism: TypeAlias = Literal["PLAIN", "GSSAPI", "OAUTHBEARER", "SCRAM-SHA-256", "SCRAM-SHA-512"]
_SecurityProtocol: TypeAlias = Literal["PLAINTEXT", "SSL", "SASL_PLAINTEXT", "SASL_SSL"]
_SocketOption: TypeAlias = tuple[int, int, int]

@type_check_only
class _KafkaProducerConfig(TypedDict, total=False):
    bootstrap_servers: _BootstrapServers
    client_id: str | None
    key_serializer: _ProducerSerializer | None
    value_serializer: _ProducerSerializer | None
    enable_idempotence: bool
    transactional_id: str | None
    transaction_timeout_ms: int
    delivery_timeout_ms: float
    acks: int | Literal["all"]
    bootstrap_topics_filter: set[str]
    compression_type: Literal["gzip", "snappy", "lz4", "zstd"] | None
    retries: int | float
    batch_size: int
    linger_ms: int
    partitioner: _Partitioner
    connections_max_idle_ms: int
    max_block_ms: int
    max_request_size: int
    allow_auto_create_topics: bool
    metadata_max_age_ms: int
    retry_backoff_ms: int
    request_timeout_ms: int
    receive_buffer_bytes: int | None
    send_buffer_bytes: int | None
    socket_options: Sequence[_SocketOption]
    sock_chunk_bytes: int
    sock_chunk_buffer_count: int
    reconnect_backoff_ms: int
    reconnect_backoff_max_ms: int
    max_in_flight_requests_per_connection: int
    security_protocol: _SecurityProtocol
    ssl_context: ssl.SSLContext | None
    ssl_check_hostname: bool
    ssl_cafile: str | None
    ssl_certfile: str | None
    ssl_keyfile: str | None
    ssl_crlfile: str | None
    ssl_password: str | None
    ssl_ciphers: str | None
    api_version: _ApiVersion | None
    api_version_auto_timeout_ms: int
    metric_reporters: Sequence[type[object]]
    metrics_enabled: bool
    metrics_num_samples: int
    metrics_sample_window_ms: int
    selector: type[selectors.BaseSelector]
    sasl_mechanism: _SaslMechanism | None
    sasl_plain_username: str | None
    sasl_plain_password: str | None
    sasl_kerberos_name: object | None
    sasl_kerberos_service_name: str
    sasl_kerberos_domain_name: str | None
    sasl_oauth_token_provider: object | None
    socks5_proxy: str | None
    kafka_client: _KafkaClientFactory

log: Incomplete
PRODUCER_CLIENT_ID_SEQUENCE: Incomplete

class KafkaProducer:
    DEFAULT_CONFIG: Incomplete
    DEPRECATED_CONFIGS: Incomplete
    config: Incomplete
    def __init__(self, **configs: Unpack[_KafkaProducerConfig]) -> None: ...
    def bootstrap_connected(self): ...
    def __del__(self) -> None: ...
    def close(self, timeout: float | None = None, null_logger: bool = False) -> None: ...
    def partitions_for(self, topic: str) -> set[int]: ...
    @classmethod
    def max_usable_produce_magic(cls, api_version): ...
    def init_transactions(self) -> None: ...
    def begin_transaction(self) -> None: ...
    def send_offsets_to_transaction(
        self, offsets: Mapping[TopicPartition, OffsetAndMetadata], consumer_group_id: str
    ) -> None: ...
    def commit_transaction(self) -> None: ...
    def abort_transaction(self) -> None: ...
    def send(
        self,
        topic: str,
        value: object = None,
        key: object = None,
        headers: Sequence[tuple[str, bytes]] | None = None,
        partition: int | None = None,
        timestamp_ms: int | None = None,
    ) -> FutureRecordMetadata: ...
    def flush(self, timeout: float | None = None) -> None: ...
    def metrics(self, raw: bool = False) -> dict[str, dict[str, object]] | dict[object, object] | None: ...



from typing import Final

FACILITY_CONTROL: Final = 0xA

def MAKE_SCODE(sev: int, fac: int, code: int) -> int: ...
def STD_CTL_SCODE(n: int) -> int: ...

CTL_E_ILLEGALFUNCTIONCALL: Final = -2146828283
CTL_E_OVERFLOW: Final = -2146828282
CTL_E_OUTOFMEMORY: Final = -2146828281
CTL_E_DIVISIONBYZERO: Final = -2146828277
CTL_E_OUTOFSTRINGSPACE: Final = -2146828274
CTL_E_OUTOFSTACKSPACE: Final = -2146828260
CTL_E_BADFILENAMEORNUMBER: Final = -2146828236
CTL_E_FILENOTFOUND: Final = -2146828235
CTL_E_BADFILEMODE: Final = -2146828234
CTL_E_FILEALREADYOPEN: Final = -2146828233
CTL_E_DEVICEIOERROR: Final = -2146828231
CTL_E_FILEALREADYEXISTS: Final = -2146828230
CTL_E_BADRECORDLENGTH: Final = -2146828229
CTL_E_DISKFULL: Final = -2146828227
CTL_E_BADRECORDNUMBER: Final = -2146828225
CTL_E_BADFILENAME: Final = -2146828224
CTL_E_TOOMANYFILES: Final = -2146828221
CTL_E_DEVICEUNAVAILABLE: Final = -2146828220
CTL_E_PERMISSIONDENIED: Final = -2146828218
CTL_E_DISKNOTREADY: Final = -2146828217
CTL_E_PATHFILEACCESSERROR: Final = -2146828213
CTL_E_PATHNOTFOUND: Final = -2146828212
CTL_E_INVALIDPATTERNSTRING: Final = -2146828195
CTL_E_INVALIDUSEOFNULL: Final = -2146828194
CTL_E_INVALIDFILEFORMAT: Final = -2146827967
CTL_E_INVALIDPROPERTYVALUE: Final = -2146827908
CTL_E_INVALIDPROPERTYARRAYINDEX: Final = -2146827907
CTL_E_SETNOTSUPPORTEDATRUNTIME: Final = -2146827906
CTL_E_SETNOTSUPPORTED: Final = -2146827905
CTL_E_NEEDPROPERTYARRAYINDEX: Final = -2146827903
CTL_E_SETNOTPERMITTED: Final = -2146827901
CTL_E_GETNOTSUPPORTEDATRUNTIME: Final = -2146827895
CTL_E_GETNOTSUPPORTED: Final = -2146827894
CTL_E_PROPERTYNOTFOUND: Final = -2146827866
CTL_E_INVALIDCLIPBOARDFORMAT: Final = -2146827828
CTL_E_INVALIDPICTURE: Final = -2146827807
CTL_E_PRINTERERROR: Final = -2146827806
CTL_E_CANTSAVEFILETOTEMP: Final = -2146827553
CTL_E_SEARCHTEXTNOTFOUND: Final = -2146827544
CTL_E_REPLACEMENTSTOOLONG: Final = -2146827542
CONNECT_E_FIRST: Final = -2147220992
CONNECT_E_LAST: Final = -2147220977
CONNECT_S_FIRST: Final = 262656
CONNECT_S_LAST: Final = 262671
CONNECT_E_NOCONNECTION: Final = -2147220992
CONNECT_E_ADVISELIMIT: Final = -2147220991
CONNECT_E_CANNOTCONNECT: Final = -2147220990
CONNECT_E_OVERRIDDEN: Final = -2147220989
CLASS_E_NOTLICENSED: Final = -2147221230



from _typeshed import Incomplete

from kafka.protocol.api import Request, Response

class FindCoordinatorResponse_v0(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class FindCoordinatorResponse_v1(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class FindCoordinatorResponse_v2(Response):
    API_KEY: int
    API_VERSION: int
    SCHEMA: Incomplete

class FindCoordinatorRequest_v0(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = FindCoordinatorResponse_v0
    SCHEMA: Incomplete

class FindCoordinatorRequest_v1(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = FindCoordinatorResponse_v1
    SCHEMA: Incomplete

class FindCoordinatorRequest_v2(Request):
    API_KEY: int
    API_VERSION: int
    RESPONSE_TYPE = FindCoordinatorResponse_v2
    SCHEMA: Incomplete

FindCoordinatorRequest: Incomplete
FindCoordinatorResponse: Incomplete



import sys
from _typeshed import Incomplete, StrOrBytesPath
from collections.abc import Callable, Collection, Iterable, Iterator
from contextlib import AbstractContextManager
from subprocess import _CMD, _ENV, _FILE
from types import TracebackType
from typing import Any, Literal, Protocol, TypeAlias, overload, type_check_only
from typing_extensions import Self, deprecated

from psutil._common import (
    AIX as AIX,
    BSD as BSD,
    CONN_CLOSE as CONN_CLOSE,
    CONN_CLOSE_WAIT as CONN_CLOSE_WAIT,
    CONN_CLOSING as CONN_CLOSING,
    CONN_ESTABLISHED as CONN_ESTABLISHED,
    CONN_FIN_WAIT1 as CONN_FIN_WAIT1,
    CONN_FIN_WAIT2 as CONN_FIN_WAIT2,
    CONN_LAST_ACK as CONN_LAST_ACK,
    CONN_LISTEN as CONN_LISTEN,
    CONN_NONE as CONN_NONE,
    CONN_SYN_RECV as CONN_SYN_RECV,
    CONN_SYN_SENT as CONN_SYN_SENT,
    CONN_TIME_WAIT as CONN_TIME_WAIT,
    FREEBSD as FREEBSD,
    LINUX as LINUX,
    MACOS as MACOS,
    NETBSD as NETBSD,
    NIC_DUPLEX_FULL as NIC_DUPLEX_FULL,
    NIC_DUPLEX_HALF as NIC_DUPLEX_HALF,
    NIC_DUPLEX_UNKNOWN as NIC_DUPLEX_UNKNOWN,
    OPENBSD as OPENBSD,
    OSX as OSX,
    POSIX as POSIX,
    POWER_TIME_UNKNOWN as POWER_TIME_UNKNOWN,
    POWER_TIME_UNLIMITED as POWER_TIME_UNLIMITED,
    STATUS_DEAD as STATUS_DEAD,
    STATUS_DISK_SLEEP as STATUS_DISK_SLEEP,
    STATUS_IDLE as STATUS_IDLE,
    STATUS_LOCKED as STATUS_LOCKED,
    STATUS_PARKED as STATUS_PARKED,
    STATUS_RUNNING as STATUS_RUNNING,
    STATUS_SLEEPING as STATUS_SLEEPING,
    STATUS_STOPPED as STATUS_STOPPED,
    STATUS_TRACING_STOP as STATUS_TRACING_STOP,
    STATUS_WAITING as STATUS_WAITING,
    STATUS_WAKING as STATUS_WAKING,
    STATUS_ZOMBIE as STATUS_ZOMBIE,
    SUNOS as SUNOS,
    WINDOWS as WINDOWS,
    AccessDenied as AccessDenied,
    Error as Error,
    NoSuchProcess as NoSuchProcess,
    TimeoutExpired as TimeoutExpired,
    ZombieProcess as ZombieProcess,
)

from . import _ntuples as _ntp

if sys.platform == "linux":
    from ._pslinux import (
        IOPRIO_CLASS_BE as IOPRIO_CLASS_BE,
        IOPRIO_CLASS_IDLE as IOPRIO_CLASS_IDLE,
        IOPRIO_CLASS_NONE as IOPRIO_CLASS_NONE,
        IOPRIO_CLASS_RT as IOPRIO_CLASS_RT,
    )
    def sensors_temperatures(fahrenheit: bool = False) -> dict[str, list[_ntp.shwtemp]]: ...
    def sensors_fans() -> dict[str, list[_ntp.sfan]]: ...
    PROCFS_PATH: str
    RLIMIT_AS: int
    RLIMIT_CORE: int
    RLIMIT_CPU: int
    RLIMIT_DATA: int
    RLIMIT_FSIZE: int
    RLIMIT_LOCKS: int
    RLIMIT_MEMLOCK: int
    RLIMIT_MSGQUEUE: int
    RLIMIT_NICE: int
    RLIMIT_NOFILE: int
    RLIMIT_NPROC: int
    RLIMIT_RSS: int
    RLIMIT_RTPRIO: int
    RLIMIT_RTTIME: int
    RLIMIT_SIGPENDING: int
    RLIMIT_STACK: int
    RLIM_INFINITY: int
if sys.platform == "win32":
    from ._psutil_windows import (
        ABOVE_NORMAL_PRIORITY_CLASS as ABOVE_NORMAL_PRIORITY_CLASS,
        BELOW_NORMAL_PRIORITY_CLASS as BELOW_NORMAL_PRIORITY_CLASS,
        HIGH_PRIORITY_CLASS as HIGH_PRIORITY_CLASS,
        IDLE_PRIORITY_CLASS as IDLE_PRIORITY_CLASS,
        NORMAL_PRIORITY_CLASS as NORMAL_PRIORITY_CLASS,
        REALTIME_PRIORITY_CLASS as REALTIME_PRIORITY_CLASS,
    )
    from ._pswindows import (
        CONN_DELETE_TCB as CONN_DELETE_TCB,
        IOPRIO_HIGH as IOPRIO_HIGH,
        IOPRIO_LOW as IOPRIO_LOW,
        IOPRIO_NORMAL as IOPRIO_NORMAL,
        IOPRIO_VERYLOW as IOPRIO_VERYLOW,
        win_service_get as win_service_get,
        win_service_iter as win_service_iter,
    )

# Linux + glibc, Windows, macOS, FreeBSD, NetBSD:
def heap_info() -> _ntp.pheap: ...
def heap_trim() -> None: ...

if sys.platform == "linux":
    from ._pslinux import sensors_battery as sensors_battery
elif sys.platform == "darwin":
    from ._psosx import sensors_battery as sensors_battery
elif sys.platform == "win32":
    from ._pswindows import sensors_battery as sensors_battery
else:
    def sensors_battery(): ...

AF_LINK: int
version_info: tuple[int, int, int]
__version__: str
__author__: str

_Status: TypeAlias = Literal[
    "running",
    "sleeping",
    "disk-sleep",
    "stopped",
    "tracing-stop",
    "zombie",
    "dead",
    "wake-kill",
    "waking",
    "idle",
    "locked",
    "waiting",
    "suspended",
    "parked",
]

class Process:
    def __init__(self, pid: int | None = None) -> None: ...
    def __eq__(self, other: object) -> bool: ...
    def __ne__(self, other: object) -> bool: ...
    def __hash__(self) -> int: ...
    @property
    def pid(self) -> int: ...
    # Only present if attrs argument is passed to process_iter
    info: dict[str, Incomplete]
    def oneshot(self) -> AbstractContextManager[None]: ...
    def as_dict(
        self, attrs: list[str] | tuple[str, ...] | set[str] | frozenset[str] | None = None, ad_value=None
    ) -> dict[str, Incomplete]: ...
    def parent(self) -> Process | None: ...
    def parents(self) -> list[Process]: ...
    def is_running(self) -> bool: ...
    def ppid(self) -> int: ...
    def name(self) -> str: ...
    def exe(self) -> str: ...
    def cmdline(self) -> list[str]: ...
    def status(self) -> _Status: ...
    def username(self) -> str: ...
    def create_time(self) -> float: ...
    def cwd(self) -> str: ...
    def nice(self, value: int | None = None) -> int: ...
    if sys.platform != "win32":
        def uids(self) -> _ntp.puids: ...
        def gids(self) -> _ntp.pgids: ...
        def terminal(self) -> str: ...
        def num_fds(self) -> int: ...
    if sys.platform != "darwin":
        def io_counters(self) -> _ntp.pio: ...
        def ionice(self, ioclass: int | None = None, value: int | None = None) -> _ntp.pionice: ...

        @overload
        def cpu_affinity(self, cpus: None = None) -> list[int]: ...
        @overload
        def cpu_affinity(self, cpus: list[int]) -> None: ...

        def memory_maps(self, grouped: bool = True) -> list[Incomplete]: ...
    if sys.platform == "linux":
        def rlimit(self, resource: int, limits: tuple[int, int] | None = None) -> tuple[int, int]: ...
        def cpu_num(self) -> int: ...

    def environ(self) -> dict[str, str]: ...
    if sys.platform == "win32":
        def num_handles(self) -> int: ...

    def num_ctx_switches(self) -> _ntp.pctxsw: ...
    def num_threads(self) -> int: ...
    def threads(self) -> list[_ntp.pthread]: ...
    def children(self, recursive: bool = False) -> list[Process]: ...
    def cpu_percent(self, interval: float | None = None) -> float: ...
    def cpu_times(self) -> _ntp.pcputimes: ...
    def memory_info(self) -> _ntp.pmem: ...
    def memory_full_info(self) -> _ntp.pfullmem: ...
    def memory_percent(self, memtype: str = "rss") -> float: ...
    def open_files(self) -> list[_ntp.popenfile]: ...
    @deprecated('use "net_connections" method instead')
    def connections(self, kind: str = "inet") -> list[_ntp.pconn]: ...
    def send_signal(self, sig: int) -> None: ...
    def suspend(self) -> None: ...
    def resume(self) -> None: ...
    def terminate(self) -> None: ...
    def kill(self) -> None: ...
    def wait(self, timeout: float | None = None) -> int: ...
    def net_connections(self, kind: str = "inet") -> list[_ntp.pconn]: ...

class Popen(Process):
    # sync with subprocess.Popen.__init__:
    if sys.version_info >= (3, 11):
        def __init__(
            self,
            args: _CMD,
            bufsize: int = -1,
            executable: StrOrBytesPath | None = None,
            stdin: _FILE | None = None,
            stdout: _FILE | None = None,
            stderr: _FILE | None = None,
            preexec_fn: Callable[[], object] | None = None,
            close_fds: bool = True,
            shell: bool = False,
            cwd: StrOrBytesPath | None = None,
            env: _ENV | None = None,
            universal_newlines: bool | None = None,
            startupinfo: Any | None = None,
            creationflags: int = 0,
            restore_signals: bool = True,
            start_new_session: bool = False,
            pass_fds: Collection[int] = (),
            *,
            text: bool | None = None,
            encoding: str | None = None,
            errors: str | None = None,
            user: str | int | None = None,
            group: str | int | None = None,
            extra_groups: Iterable[str | int] | None = None,
            umask: int = -1,
            pipesize: int = -1,
            process_group: int | None = None,
        ) -> None: ...
    else:
        def __init__(
            self,
            args: _CMD,
            bufsize: int = -1,
            executable: StrOrBytesPath | None = None,
            stdin: _FILE | None = None,
            stdout: _FILE | None = None,
            stderr: _FILE | None = None,
            preexec_fn: Callable[[], object] | None = None,
            close_fds: bool = True,
            shell: bool = False,
            cwd: StrOrBytesPath | None = None,
            env: _ENV | None = None,
            universal_newlines: bool | None = None,
            startupinfo: Any | None = None,
            creationflags: int = 0,
            restore_signals: bool = True,
            start_new_session: bool = False,
            pass_fds: Collection[int] = (),
            *,
            text: bool | None = None,
            encoding: str | None = None,
            errors: str | None = None,
            user: str | int | None = None,
            group: str | int | None = None,
            extra_groups: Iterable[str | int] | None = None,
            umask: int = -1,
            pipesize: int = -1,
        ) -> None: ...

    def __enter__(self) -> Self: ...
    def __exit__(
        self, exc_type: type[BaseException] | None, value: BaseException | None, traceback: TracebackType | None
    ) -> None: ...
    def __getattribute__(self, name: str) -> Any: ...
    def __dir__(self) -> list[str]: ...

@type_check_only
class _ProcessIterCallable(Protocol):
    def __call__(
        self, attrs: list[str] | tuple[str, ...] | set[str] | frozenset[str] | None = None, ad_value=None
    ) -> Iterator[Process]: ...
    def cache_clear(self) -> None: ...

def pids() -> list[int]: ...
def pid_exists(pid: int) -> bool: ...

process_iter: _ProcessIterCallable

def wait_procs(
    procs: Iterable[Process], timeout: float | None = None, callback: Callable[[Process], object] | None = None
) -> tuple[list[Process], list[Process]]: ...
def cpu_count(logical: bool = True) -> int | None: ...

@overload
def cpu_freq(percpu: Literal[False] = False) -> _ntp.scpufreq: ...
@overload
def cpu_freq(percpu: Literal[True]) -> list[_ntp.scpufreq]: ...

@overload
def cpu_times(percpu: Literal[False] = False) -> _ntp.scputimes: ...
@overload
def cpu_times(percpu: Literal[True]) -> list[_ntp.scputimes]: ...

@overload
def cpu_percent(interval: float | None = None, percpu: Literal[False] = False) -> float: ...
@overload
def cpu_percent(interval: float | None, percpu: Literal[True]) -> list[float]: ...
@overload
def cpu_percent(*, percpu: Literal[True]) -> list[float]: ...

@overload
def cpu_times_percent(interval: float | None = None, percpu: Literal[False] = False) -> _ntp.scputimes: ...
@overload
def cpu_times_percent(interval: float | None, percpu: Literal[True]) -> list[_ntp.scputimes]: ...
@overload
def cpu_times_percent(*, percpu: Literal[True]) -> list[_ntp.scputimes]: ...

def cpu_stats() -> _ntp.scpustats: ...
def getloadavg() -> tuple[float, float, float]: ...
def virtual_memory() -> _ntp.svmem: ...
def swap_memory() -> _ntp.sswap: ...
def disk_usage(path: str) -> _ntp.sdiskusage: ...
def disk_partitions(all: bool = False) -> list[_ntp.sdiskpart]: ...

# TODO: Incorrect sdiskio for BSD systems:
@overload
def disk_io_counters(perdisk: Literal[False] = False, nowrap: bool = True) -> _ntp.sdiskio | None: ...
@overload
def disk_io_counters(perdisk: Literal[True], nowrap: bool = True) -> dict[str, _ntp.sdiskio]: ...

@overload
def net_io_counters(pernic: Literal[False] = False, nowrap: bool = True) -> _ntp.snetio: ...
@overload
def net_io_counters(pernic: Literal[True], nowrap: bool = True) -> dict[str, _ntp.snetio]: ...

def net_connections(kind: str = "inet") -> list[_ntp.sconn]: ...
def net_if_addrs() -> dict[str, list[_ntp.snicaddr]]: ...
def net_if_stats() -> dict[str, _ntp.snicstats]: ...
def boot_time() -> float: ...
def users() -> list[_ntp.suser]: ...



from _typeshed import Incomplete

from MySQLdb import connections as connections, constants as constants, converters as converters, cursors as cursors
from MySQLdb._mysql import (
    DatabaseError as DatabaseError,
    DataError as DataError,
    Error as Error,
    IntegrityError as IntegrityError,
    InterfaceError as InterfaceError,
    InternalError as InternalError,
    MySQLError as MySQLError,
    NotSupportedError as NotSupportedError,
    OperationalError as OperationalError,
    ProgrammingError as ProgrammingError,
    Warning as Warning,
    debug as debug,
    get_client_info as get_client_info,
    string_literal as string_literal,
)
from MySQLdb.connections import Connection as Connection
from MySQLdb.constants import FIELD_TYPE as FIELD_TYPE
from MySQLdb.release import version_info as version_info
from MySQLdb.times import (
    Date as Date,
    DateFromTicks as DateFromTicks,
    Time as Time,
    TimeFromTicks as TimeFromTicks,
    Timestamp as Timestamp,
    TimestampFromTicks as TimestampFromTicks,
)

threadsafety: int
apilevel: str
paramstyle: str

class DBAPISet(frozenset[Incomplete]):
    def __eq__(self, other): ...

STRING: Incomplete
BINARY: Incomplete
NUMBER: Incomplete
DATE: Incomplete
TIME: Incomplete
TIMESTAMP: Incomplete
DATETIME: Incomplete
ROWID: Incomplete

def Binary(x): ...
def Connect(*args, **kwargs) -> Connection: ...

connect = Connect

__all__ = [
    "BINARY",
    "Binary",
    "Connect",
    "Connection",
    "DATE",
    "Date",
    "Time",
    "Timestamp",
    "DateFromTicks",
    "TimeFromTicks",
    "TimestampFromTicks",
    "DataError",
    "DatabaseError",
    "Error",
    "FIELD_TYPE",
    "IntegrityError",
    "InterfaceError",
    "InternalError",
    "MySQLError",
    "NUMBER",
    "NotSupportedError",
    "DBAPISet",
    "OperationalError",
    "ProgrammingError",
    "ROWID",
    "STRING",
    "TIME",
    "TIMESTAMP",
    "Warning",
    "apilevel",
    "connect",
    "connections",
    "constants",
    "converters",
    "cursors",
    "debug",
    "get_client_info",
    "paramstyle",
    "string_literal",
    "threadsafety",
    "version_info",
]



import enum
import sys
from _typeshed import FileDescriptorOrPath, Incomplete, StrOrBytesPath, Unused
from collections.abc import Callable

from . import _ntuples as ntp

def pid_exists(pid: int) -> bool: ...

# Sync with `signal.Signals`, but with opposite values:
class Negsignal(enum.IntEnum):
    SIGABRT = -6
    SIGFPE = -8
    SIGILL = -4
    SIGINT = -2
    SIGSEGV = -11
    SIGTERM = -15

    if sys.platform == "win32":
        SIGBREAK = -21
        CTRL_C_EVENT = 0
        CTRL_BREAK_EVENT = -1
    else:
        SIGALRM = -14
        SIGBUS = -7
        SIGCHLD = -17
        SIGCONT = -18
        SIGHUP = -1
        SIGIO = -29
        SIGIOT = -6
        SIGKILL = -9
        SIGPIPE = -13
        SIGPROF = -27
        SIGQUIT = -3
        SIGSTOP = -19
        SIGSYS = -31
        SIGTRAP = -5
        SIGTSTP = -20
        SIGTTIN = -21
        SIGTTOU = -22
        SIGURG = -23
        SIGUSR1 = -10
        SIGUSR2 = -12
        SIGVTALRM = -26
        SIGWINCH = -28
        SIGXCPU = -24
        SIGXFSZ = -25
        if sys.platform != "linux":
            SIGEMT = -7
            SIGINFO = -29
        if sys.platform != "darwin":
            SIGCLD = -17
            SIGPOLL = -29
            SIGPWR = -30
            SIGRTMAX = -64
            SIGRTMIN = -34
            if sys.version_info >= (3, 11):
                SIGSTKFLT = -16

def negsig_to_enum(num: int) -> int: ...
def convert_exit_code(status: int) -> int: ...
def wait_pid_posix(
    pid: int,
    timeout: float | None = None,
    _waitpid: Unused = ...,
    _timer: Callable[[], float] = ...,
    _min: Callable[..., Incomplete] = ...,
    _sleep: Callable[[float], None] = ...,
    _pid_exists: Callable[[int], bool] = ...,
) -> int | None: ...
def wait_pid_pidfd_open(pid: int, timeout: float | None = None) -> int | None: ...
def wait_pid_kqueue(pid: int, timeout: float | None = None) -> int | None: ...
def can_use_pidfd_open() -> bool: ...
def can_use_kqueue() -> bool: ...
def wait_pid(pid: int, timeout: float | None = None) -> int | None: ...

if sys.platform == "darwin":
    def disk_usage(path: StrOrBytesPath) -> ntp.sdiskusage: ...

else:
    def disk_usage(path: FileDescriptorOrPath) -> ntp.sdiskusage: ...

def get_terminal_map() -> dict[int, str]: ...

__all__ = ["pid_exists", "wait_pid", "disk_usage", "get_terminal_map"]



import abc
import threading
from _typeshed import Incomplete
from typing import ClassVar

from kafka import errors as Errors

log: Incomplete
heartbeat_log: Incomplete

class MemberState:
    UNJOINED: str
    REBALANCING: str
    STABLE: str

class Generation:
    NO_GENERATION: ClassVar[Generation]
    generation_id: Incomplete
    member_id: Incomplete
    protocol: Incomplete
    def __init__(self, generation_id, member_id, protocol) -> None: ...
    def has_member_id(self): ...
    def __eq__(self, other): ...

class UnjoinedGroupException(Errors.KafkaError):
    retriable: bool

class BaseCoordinator(metaclass=abc.ABCMeta):
    DEFAULT_CONFIG: Incomplete
    config: Incomplete
    heartbeat: Incomplete
    rejoin_needed: bool
    rejoining: bool
    state: Incomplete
    join_future: Incomplete
    coordinator_id: Incomplete
    def __init__(self, client, **configs) -> None: ...
    @property
    def group_id(self): ...
    @property
    def group_instance_id(self): ...
    @abc.abstractmethod
    def protocol_type(self): ...
    @abc.abstractmethod
    def group_protocols(self): ...
    def coordinator_unknown(self): ...
    def coordinator(self): ...
    def connected(self): ...
    def ensure_coordinator_ready(self, timeout_ms=None): ...
    def lookup_coordinator(self): ...
    def need_rejoin(self): ...
    def poll_heartbeat(self) -> None: ...
    def time_to_next_heartbeat(self): ...
    def ensure_active_group(self, timeout_ms=None): ...
    def join_group(self, timeout_ms=None): ...
    def coordinator_dead(self, error) -> None: ...
    def generation_if_stable(self): ...
    def generation(self): ...
    def rebalance_in_progress(self): ...
    def reset_generation(self, member_id="") -> None: ...
    def request_rejoin(self) -> None: ...
    def __del__(self) -> None: ...
    def close(self, timeout_ms=None) -> None: ...
    def is_dynamic_member(self): ...
    def maybe_leave_group(self, timeout_ms=None) -> None: ...

class GroupCoordinatorMetrics:
    heartbeat: Incomplete
    metrics: Incomplete
    metric_group_name: Incomplete
    heartbeat_latency: Incomplete
    join_latency: Incomplete
    sync_latency: Incomplete
    def __init__(self, heartbeat, metrics, prefix, tags=None) -> None: ...

class HeartbeatThread(threading.Thread):
    name: Incomplete
    coordinator: Incomplete
    enabled: bool
    closed: bool
    failed: Incomplete
    def __init__(self, coordinator) -> None: ...
    def enable(self) -> None: ...
    def disable(self) -> None: ...
    def close(self, timeout_ms=None) -> None: ...
    def run(self) -> None: ...



from _typeshed import (
    FileDescriptorOrPath,
    OpenBinaryMode,
    OpenBinaryModeReading,
    OpenBinaryModeUpdating,
    OpenBinaryModeWriting,
    OpenTextMode,
)
from asyncio import AbstractEventLoop
from collections.abc import Callable
from concurrent.futures import Executor
from functools import _SingleDispatchCallable
from typing import Any, Literal, TypeAlias, overload

from ..base import AiofilesContextManager
from .binary import AsyncBufferedIOBase, AsyncBufferedReader, AsyncFileIO, AsyncIndirectBufferedIOBase, _UnknownAsyncBinaryIO
from .text import AsyncTextIndirectIOWrapper, AsyncTextIOWrapper

_Opener: TypeAlias = Callable[[str, int], int]

# Text mode: always returns AsyncTextIOWrapper
@overload
def open(
    file: FileDescriptorOrPath,
    mode: OpenTextMode = "r",
    buffering: int = -1,
    encoding: str | None = None,
    errors: str | None = None,
    newline: str | None = None,
    closefd: bool = True,
    opener: _Opener | None = None,
    *,
    loop: AbstractEventLoop | None = None,
    executor: Executor | None = None,
) -> AiofilesContextManager[AsyncTextIOWrapper]: ...

# Unbuffered binary: returns a FileIO
@overload
def open(
    file: FileDescriptorOrPath,
    mode: OpenBinaryMode,
    buffering: Literal[0],
    encoding: None = None,
    errors: None = None,
    newline: None = None,
    closefd: bool = True,
    opener: _Opener | None = None,
    *,
    loop: AbstractEventLoop | None = None,
    executor: Executor | None = None,
) -> AiofilesContextManager[AsyncFileIO]: ...

# Buffered binary reading/updating: AsyncBufferedReader
@overload
def open(
    file: FileDescriptorOrPath,
    mode: OpenBinaryModeReading | OpenBinaryModeUpdating,
    buffering: Literal[-1, 1] = -1,
    encoding: None = None,
    errors: None = None,
    newline: None = None,
    closefd: bool = True,
    opener: _Opener | None = None,
    *,
    loop: AbstractEventLoop | None = None,
    executor: Executor | None = None,
) -> AiofilesContextManager[AsyncBufferedReader]: ...

# Buffered binary writing: AsyncBufferedIOBase
@overload
def open(
    file: FileDescriptorOrPath,
    mode: OpenBinaryModeWriting,
    buffering: Literal[-1, 1] = -1,
    encoding: None = None,
    errors: None = None,
    newline: None = None,
    closefd: bool = True,
    opener: _Opener | None = None,
    *,
    loop: AbstractEventLoop | None = None,
    executor: Executor | None = None,
) -> AiofilesContextManager[AsyncBufferedIOBase]: ...

# Buffering cannot be determined: fall back to _UnknownAsyncBinaryIO
@overload
def open(
    file: FileDescriptorOrPath,
    mode: OpenBinaryMode,
    buffering: int = -1,
    encoding: None = None,
    errors: None = None,
    newline: None = None,
    closefd: bool = True,
    opener: _Opener | None = None,
    *,
    loop: AbstractEventLoop | None = None,
    executor: Executor | None = None,
) -> AiofilesContextManager[_UnknownAsyncBinaryIO]: ...

wrap: _SingleDispatchCallable[Any]

stdin: AsyncTextIndirectIOWrapper
stdout: AsyncTextIndirectIOWrapper
stderr: AsyncTextIndirectIOWrapper
stdin_bytes: AsyncIndirectBufferedIOBase
stdout_bytes: AsyncIndirectBufferedIOBase
stderr_bytes: AsyncIndirectBufferedIOBase

__all__ = ("open", "stdin", "stdout", "stderr", "stdin_bytes", "stdout_bytes", "stderr_bytes")



from _typeshed import Incomplete
from collections.abc import Callable, Generator, Hashable, Iterable
from typing import Any

__all__ = ["ISMAGS"]

def are_all_equal(iterable: Iterable[Any]) -> bool: ...
def make_partition(
    items: Iterable[Hashable], test: Callable[[Hashable, Hashable], bool], check: bool = True
) -> list[set[Incomplete]]: ...
def node_to_part_ID_dict(partition: Iterable[Iterable[Incomplete]]) -> dict[Incomplete, int]: ...
def color_degree_by_node(G, n_colors, e_colors): ...

class EdgeLookup:
    edge_dict: Incomplete
    def __init__(self, edge_dict) -> None: ...
    def __getitem__(self, edge): ...
    def items(self): ...

class ISMAGS:
    graph: Incomplete
    subgraph: Incomplete

    def __init__(self, graph, subgraph, node_match=None, edge_match=None, cache=None) -> None: ...
    def create_aligned_partitions(self, thing_matcher, sg_things, g_things): ...
    def find_isomorphisms(self, symmetry: bool = True) -> Generator[Incomplete, Incomplete, Incomplete]: ...
    def largest_common_subgraph(self, symmetry: bool = True) -> Generator[Incomplete, Incomplete]: ...
    def analyze_subgraph_symmetry(self) -> dict[Hashable, set[Hashable]]: ...
    def is_isomorphic(self, symmetry: bool = False) -> bool: ...
    def subgraph_is_isomorphic(self, symmetry: bool = False) -> bool: ...
    def isomorphisms_iter(self, symmetry: bool = True) -> Generator[Incomplete, Incomplete]: ...
    def subgraph_isomorphisms_iter(self, symmetry: bool = True): ...



from _typeshed import Incomplete
from typing import NamedTuple

from kafka.future import Future
from kafka.structs import TopicPartition

class FutureProduceResult(Future):
    topic_partition: TopicPartition
    def __init__(self, topic_partition: TopicPartition) -> None: ...
    def success(self, value): ...
    def failure(self, error): ...
    def wait(self, timeout=None): ...

class FutureRecordMetadata(Future):
    args: Incomplete
    def __init__(
        self,
        produce_future,
        batch_index,
        timestamp_ms,
        checksum,
        serialized_key_size,
        serialized_value_size,
        serialized_header_size,
    ) -> None: ...
    def get(self, timeout=None): ...

class RecordMetadata(NamedTuple):
    topic: str
    partition: int
    topic_partition: TopicPartition
    offset: int
    timestamp: int
    checksum: int | None
    serialized_key_size: int
    serialized_value_size: int
    serialized_header_size: int



from _typeshed import Incomplete
from collections.abc import Callable

def register_temporary_credential_hooks(authorization_server, cache, key_prefix: str = "temporary_credential:") -> None: ...
def create_exists_nonce_func(
    cache, key_prefix="nonce:", expires=86400
) -> Callable[[Incomplete, Incomplete, Incomplete, Incomplete], Incomplete]: ...
def register_nonce_hooks(authorization_server, cache, key_prefix: str = "nonce:", expires=86400) -> None: ...



class Quota:
    def __init__(self, bound, is_upper) -> None: ...
    @staticmethod
    def upper_bound(upper_bound): ...
    @staticmethod
    def lower_bound(lower_bound): ...
    def is_upper_bound(self): ...
    @property
    def bound(self): ...
    def is_acceptable(self, value): ...
    def __hash__(self): ...
    def __eq__(self, other): ...
    def __ne__(self, other): ...



import abc

class Serializer(metaclass=abc.ABCMeta):
    __meta__ = abc.ABCMeta
    def __init__(self, **config) -> None: ...
    @abc.abstractmethod
    def serialize(self, topic, value): ...
    def close(self) -> None: ...

class Deserializer(metaclass=abc.ABCMeta):
    __meta__ = abc.ABCMeta
    def __init__(self, **config) -> None: ...
    @abc.abstractmethod
    def deserialize(self, topic, bytes_): ...
    def close(self) -> None: ...



from _typeshed import Incomplete
from collections import OrderedDict
from collections.abc import Mapping, Sized
from re import Pattern
from typing import Final, TypeVar

from dateparser.conf import Settings

_K = TypeVar("_K", bound=Sized)
_V = TypeVar("_V")

NUMERAL_PATTERN: Final[Pattern[str]]

class Locale:
    shortname: str
    info: OrderedDict[str, Incomplete]
    def __init__(self, shortname: str, language_info: Mapping[Incomplete, Incomplete]) -> None: ...
    def is_applicable(self, date_string: str, strip_timezone: bool = False, settings: Settings | None = None) -> bool: ...
    def count_applicability(self, text: str, strip_timezone: bool = False, settings: Settings | None = None) -> list[int]: ...
    @staticmethod
    def clean_dictionary(dictionary: Mapping[_K, _V], threshold: int = 2) -> Mapping[_K, _V]: ...
    def translate(self, date_string: str, keep_formatting: bool = False, settings: Settings | None = None) -> str: ...
    def translate_search(self, search_string: str, settings: Settings | None = None) -> tuple[list[str], list[str]]: ...
    def get_wordchars_for_detection(self, settings: Settings) -> set[str]: ...
    def to_parserinfo(self, base_cls: type = ...): ...



from google.protobuf.descriptor import Descriptor, FieldDescriptor as FieldDescriptor
from google.protobuf.message import Message

class FieldMask:
    def ToJsonString(self) -> str: ...
    def FromJsonString(self, value: str) -> None: ...
    def IsValidForDescriptor(self, message_descriptor: Descriptor) -> bool: ...
    def AllFieldsFromDescriptor(self, message_descriptor: Descriptor) -> None: ...
    def CanonicalFormFromMask(self, mask: FieldMask) -> None: ...
    def Union(self, mask1: FieldMask, mask2: FieldMask) -> None: ...
    def Intersect(self, mask1: FieldMask, mask2: FieldMask) -> None: ...
    def MergeMessage(
        self, source: Message, destination: Message, replace_message_field: bool = False, replace_repeated_field: bool = False
    ) -> None: ...



from typing import Final

from google.protobuf.descriptor_pb2 import FileDescriptorProto

__author__: Final[str]

class Error(Exception): ...
class DescriptorDatabaseConflictingDefinitionError(Error): ...

class DescriptorDatabase:
    def __init__(self) -> None: ...
    def Add(self, file_desc_proto: FileDescriptorProto) -> None: ...
    def FindFileByName(self, name: str) -> FileDescriptorProto: ...
    def FindFileContainingSymbol(self, symbol: str) -> FileDescriptorProto: ...
    def FindFileContainingExtension(self, extendee_name: str, extension_number: int) -> FileDescriptorProto | None: ...
    def FindAllExtensionNumbers(self, extendee_name: str) -> list[int]: ...



from _typeshed import Incomplete

def format_workers(data: dict[str, Incomplete]) -> str: ...
def format_dirty(data: dict[str, Incomplete]) -> str: ...
def format_stats(data: dict[str, Incomplete]) -> str: ...
def format_listeners(data: dict[str, Incomplete]) -> str: ...
def format_config(data: dict[str, Incomplete]) -> str: ...
def format_help(data: dict[str, Incomplete]) -> str: ...
def format_all(data: dict[str, Incomplete]) -> str: ...
def format_response(command: str, data: dict[str, Incomplete]) -> str: ...
def run_command(socket_path: str, command: str, json_output: bool = False) -> int: ...
def run_interactive(socket_path: str, json_output: bool = False) -> int: ...
def main() -> int: ...



from _typeshed import Incomplete
from typing import NamedTuple

log: Incomplete

class ConsumerPair(NamedTuple):
    src_member_id: Incomplete
    dst_member_id: Incomplete

def is_sublist(source, target): ...

class PartitionMovements:
    partition_movements_by_topic: Incomplete
    partition_movements: Incomplete
    def __init__(self) -> None: ...
    def move_partition(self, partition, old_consumer, new_consumer) -> None: ...
    def get_partition_to_be_moved(self, partition, old_consumer, new_consumer): ...
    def are_sticky(self): ...



from _typeshed import ReadableBuffer
from collections.abc import Iterable
from typing import Literal, SupportsBytes, SupportsIndex, overload

from cryptography.hazmat.primitives.asymmetric.types import PrivateKeyTypes, PublicKeyTypes
from cryptography.hazmat.primitives.serialization.ssh import SSHPublicKeyTypes

@overload  # if ssh_type is None
def load_pem_key(
    raw: str | bytes | float | Iterable[SupportsIndex] | SupportsIndex | SupportsBytes | ReadableBuffer,
    ssh_type: None = None,
    key_type: str | None = None,
    password: bytes | None = None,
) -> PublicKeyTypes | PrivateKeyTypes: ...
@overload  # if key_type == "public"
def load_pem_key(
    raw: str | bytes | float | Iterable[SupportsIndex] | SupportsIndex | SupportsBytes | ReadableBuffer,
    ssh_type: ReadableBuffer | tuple[ReadableBuffer, ...] | None = None,
    key_type: Literal["public"] = ...,
    password: bytes | None = None,
) -> PublicKeyTypes: ...
@overload  # if key_type is not empty, but not "public"
def load_pem_key(
    raw: str | bytes | float | Iterable[SupportsIndex] | SupportsIndex | SupportsBytes | ReadableBuffer,
    ssh_type: ReadableBuffer | tuple[ReadableBuffer, ...] | None = None,
    key_type: str = ...,
    password: bytes | None = None,
) -> PrivateKeyTypes: ...
@overload  # if ssh_type is not empty
def load_pem_key(
    raw: str | bytes | float | Iterable[SupportsIndex] | SupportsIndex | SupportsBytes | ReadableBuffer,
    ssh_type: ReadableBuffer | tuple[ReadableBuffer, ...] = ...,
    key_type: str | None = None,
    password: bytes | None = None,
) -> SSHPublicKeyTypes | PublicKeyTypes | PrivateKeyTypes: ...



from collections.abc import Callable, Iterable
from pathlib import PurePath
from types import TracebackType
from typing import Final, Literal, Protocol, TypeAlias, type_check_only
from typing_extensions import Self

from paramiko.channel import Channel
from paramiko.transport import Transport

__version__: Final[str]

SCP_COMMAND: Final = b"scp"
PATH_TYPES: Final[tuple[type[str], type[bytes], type[PurePath]]]
bytes_sep: Final[bytes]

PathTypes: TypeAlias = str | bytes | PurePath

@type_check_only
class _PutFOReader(Protocol):
    def read(self, size: int, /) -> str | bytes | bytearray: ...
    def tell(self) -> int: ...
    def seek(self, offset: int, whence: Literal[0, 2], /) -> object: ...

def asbytes(s: PathTypes) -> bytes: ...
def asunicode(s: bytes | str) -> str: ...
def asunicode_win(s: bytes | str) -> str: ...

class SCPClient:
    transport: Transport
    buff_size: int
    socket_timeout: float | None
    channel: Channel | None
    preserve_times: bool
    sanitize: Callable[[bytes], bytes]
    peername: tuple[str, int]
    scp_command: bytes
    def __init__(
        self,
        transport: Transport,
        buff_size: int = 16384,
        socket_timeout: float | None = 10.0,
        progress: Callable[[str | bytes, int, int], None] | None = None,
        progress4: Callable[[str | bytes, int, int, tuple[str, int]], None] | None = None,
        sanitize: Callable[[bytes], bytes] | Literal[False] = ...,
        limit_bw: int | None = None,
    ) -> None: ...
    def __enter__(self) -> Self: ...
    def __exit__(
        self, type: type[BaseException] | None, value: BaseException | None, traceback: TracebackType | None
    ) -> None: ...
    def put(
        self,
        files: PathTypes | Iterable[PathTypes],
        remote_path: PathTypes = b".",
        recursive: bool = False,
        preserve_times: bool = False,
    ) -> None: ...
    def putfo(self, fl: _PutFOReader, remote_path: PathTypes, mode: str | bytes = "0644", size: int | None = None) -> None: ...
    def get(
        self,
        remote_path: PathTypes | Iterable[PathTypes],
        local_path: PathTypes = "",
        recursive: bool = False,
        preserve_times: bool = False,
    ) -> None: ...
    def close(self) -> None: ...

class SCPException(Exception): ...

def put(
    transport: Transport,
    files: PathTypes | Iterable[PathTypes],
    remote_path: PathTypes = b".",
    recursive: bool = False,
    preserve_times: bool = False,
) -> None: ...
def get(
    transport: Transport,
    remote_path: PathTypes | Iterable[PathTypes],
    local_path: PathTypes = "",
    recursive: bool = False,
    preserve_times: bool = False,
) -> None: ...



class MetricName:
    def __init__(self, name, group, description=None, tags=None) -> None: ...
    @property
    def name(self): ...
    @property
    def group(self): ...
    @property
    def description(self): ...
    @property
    def tags(self): ...
    def __hash__(self): ...
    def __eq__(self, other): ...
    def __ne__(self, other): ...
